from __future__ import print_function
from __future__ import absolute_import
from builtins import range
# http://sphinx.pocoo.org/ext/math.html#

# can reuse most of rst module:
from .rst import *
from .common import align2equations, online_python_tutor, \
     get_legal_pygments_lexers, has_custom_pygments_lexer
from .misc import option, _abort
from .doconce import errwarn
# used in sphinx_dir
import time, shutil, glob, re
from doconce.misc import system, load_preprocessed_doconce_file

# RunestoneInteractive book counters
question_counter = 0
video_counter = 0

edit_markup_warning = False

def sphinx_figure(m):
    result = ''
    # m is a MatchObject

    filename = m.group('filename')
    caption = m.group('caption').strip()

    # Stubstitute DocOnce label by rst label in caption
    # (also, remove final period in caption since caption is used as hyperlink
    # text to figures).

    m_label = re.search(r'label\{(.+?)\}', caption)
    if m_label:
        label = m_label.group(1)
        result += '\n.. _%s:\n' % label
        # remove . at the end of the caption text
        parts = caption.split('label')
        parts[0] = parts[0].rstrip()
        if parts[0] and parts[0][-1] == '.':
            parts[0] = parts[0][:-1]
        parts[0] = parts[0].strip()
        # insert emphasize marks if not latex $ at the
        # beginning or end (math subst does not work for *$I=1$*)
        # or if not boldface or emphasize already in the caption
        caption_font = option('sphinx_figure_captions=', 'emphasize')
        if parts[0] and \
           caption_font == 'emphasize' and \
           not parts[0].startswith('$') and \
           not parts[0].endswith('$') and \
           not '*' in parts[0] and \
           not '_' in parts[0]:
            parts[0] = '*' + parts[0] + '*'
        #caption = '  label'.join(parts)
        caption = parts[0]
        # contrary to rst_figure, we do not write label into caption
        # since we just want to remove the whole label as part of
        # the caption (otherwise done when handling ref and label)

    else:
        if caption and caption[-1] == '.':
            caption = caption[:-1]

    # math is ignored in references to figures, test for math only
    if caption.startswith('$') and caption.endswith('$'):
        errwarn('*** warning: math only in sphinx figure caption (it will be ignored by sphinx, resulting in empty caption)\n  %s\n    FIGURE: [%s' % (caption, filename))

    #stem = os.path.splitext(filename)[0]
    #result += '\n.. figure:: ' + stem + '.*\n'  # utilize flexibility  # does not work yet
    result += '\n.. figure:: ' + filename + '\n'
    opts = m.group('options')
    if opts:
        # opts: width=600 frac=0.5 align=center
        # opts: width=600, frac=0.5, align=center
        info = [s.split('=') for s in opts.split()]
        fig_info = ['   :%s: %s' % (opt, value.replace(',', ''))
                    for opt, value in info
                    if opt not in ['frac', 'sidecap']]
        result += '\n'.join(fig_info)
    if caption:
        result += '\n\n   ' + caption + '\n'
    else:
        result += '\n\n'
    #errwarn('sphinx figure: caption=\n', caption, '\nresult:\n', result)
    return result

def sphinx_movie(m):
    filename = m.group('filename')
    special_movie = '*' in filename or '->' in filename or 'youtu.be' in filename or 'youtube.com' in filename or 'vimeo.com' in filename
    if option('runestone') and not special_movie:
        # Use RunestoneInteractive video environment
        global video_counter
        video_counter += 1
        text = """
.. video:: video_%d
   :controls:

   %s
""" % (video_counter, filename)
        return text
    else:
        # Use plain html code
        return rst_movie(m)


def sphinx_quiz_runestone(quiz):
    quiz_feedback = option('quiz_explanations=', 'on')

    text = ''
    if 'new page' in quiz:
        text += '.. !split\n%s\n%s' % (quiz['new page'], '-'*len(quiz['new page']))

    text += '.. begin quiz\n\n'
    global question_counter
    question_counter += 1
    # Multiple correct answers?
    if sum([1 for choice in quiz['choices'] if choice[0] == 'right']) > 1:
        text += '.. mchoicema:: question_%d' % question_counter + '\n'
    else:
        text += '.. mchoicemf:: question_%d' % question_counter + '\n'

    def fix_text(s, tp='answer'):
        """
        Answers and feedback in RunestoneInteractive book quizzes
        cannot contain math, figure and rst markup. Perform fixes.
        """
        drop = False
        if 'math::' in s:
            errwarn('\n*** warning: quiz %s with math block not supported:' % tp)
            errwarn(s)
            drop = True
        if '.. code-block::' in s:
            errwarn('\n*** warning: quiz %s with code block not supported:' % tp)
            errwarn(s)
            drop = True
        if '.. figure::' in s:
            errwarn('\n*** warning: quiz %s with figure not supported:' % tp)
            errwarn(s)
            drop = True
        if drop:
            return ''
        # Make multi-line paragraph a one-liner
        s = ' '.join(s.splitlines()).rstrip()
        # Fixes
        pattern = r'`(.+?) (<https?.+?)>`__'  # URL
        s = re.sub(pattern, '<a href="\g<2>"> \g<1> </a>', s)
        pattern = r'``(.+?)``'  # verbatim
        s = re.sub(pattern, '<tt>\g<1></tt>', s)
        pattern = r':math:`(.+?)`'  # inline math
        s = re.sub(pattern, '<em>\g<1></em>', s)  # mimic italic....
        pattern = r':\*(.+?)\*'  # emphasize
        s = re.sub(pattern, '\g<1>', s, flags=re.DOTALL)
        return s

    import string
    correct = []
    for i, choice in enumerate(quiz['choices']):
        if i > 4:  # not supported
            errwarn('*** warning: quiz with %d choices gets truncated (first 5)' % len(quiz['choices']))
            break
        letter = string.ascii_lowercase[i]
        text += '   :answer_%s: ' % letter
        answer = fix_text(choice[1], tp='answer')
        if not answer:
            answer = 'Too advanced typesetting prevents the text from being rendered'
        text += answer + '\n'
        if choice[0] == 'right':
            correct.append(letter)
    if correct:
        text += '   :correct: ' + ', '.join(correct) + '\n'
    else:
        errwarn('*** error: correct choice in quiz has index > 5 (max 5 allowed for RunestoneInteractive books)')
        errwarn(quiz['question'])
        _abort()
    for i, choice in enumerate(quiz['choices']):
        if i > 4:  # not supported
            break
        letter = string.ascii_lowercase[i]
        text += '   :feedback_%s: ' % letter  # must be present
        if len(choice) == 3 and quiz_feedback == 'on':
            feedback = fix_text(choice[2], tp='explanation')
            if not feedback:
                feedback = '(Too advanced typesetting prevents the text from being rendered)'
            text += feedback
        text += '\n'

    text += '\n' + indent_lines(quiz['question'], 'sphinx', ' '*3) + '\n\n\n'
    return text

def sphinx_quiz(quiz):
    if option('runestone'):
        return sphinx_quiz_runestone(quiz)
    else:
        return rst_quiz(quiz)


from .latex import fix_latex_command_regex as fix_latex

def sphinx_code(filestr, code_blocks, code_block_types,
                tex_blocks, format):
    # In rst syntax, code blocks are typeset with :: (verbatim)
    # followed by intended blocks. This function indents everything
    # inside code (or TeX) blocks.

    # default mappings of !bc environments and pygments languages:
    envir2pygments = dict(
        cod='python', pro='python',
        pycod='python', cycod='cython',
        pypro='python', cypro='cython',
        fcod='fortran', fpro='fortran',
        ccod='c', cppcod='c++',
        cpro='c', cpppro='c++',
        mcod='matlab', mpro='matlab',
        plcod='perl', plpro='perl',
        shcod='bash', shpro='bash',
        rbcod='ruby', rbpro='ruby',
        #sys='console',
        sys='text',
        rst='rst',
        css='css', csspro='css', csscod='css',
        dat='text', csv='text', txt='text',
        cc='text', ccq='text',  # not possible with extra indent for ccq
        ipy='ipy',
        xmlcod='xml', xmlpro='xml', xml='xml',
        htmlcod='html', htmlpro='html', html='html',
        texcod='latex', texpro='latex', tex='latex',
        latexcod='latex', latexpro='latex', latex='latex',
        do='doconce',
        pyshell='python',
        pyoptpro='python', pyscpro='python',
        )

    # grab line with: # sphinx code-blocks: cod=python cpp=c++ etc
    # (do this before code is inserted in case verbatim blocks contain
    # such specifications for illustration)
    m = re.search(r'.. *[Ss]phinx +code-blocks?:(.+)', filestr)
    if m:
        defs_line = m.group(1)
        # turn specifications into a dictionary:
        for definition in defs_line.split():
            key, value = definition.split('=')
            envir2pygments[key] = value

    # First indent all code blocks

    for i in range(len(code_blocks)):
        if code_block_types[i].startswith('pyoptpro') and not option('runestone'):
            code_blocks[i] = online_python_tutor(code_blocks[i],
                                                 return_tp='iframe')
        if code_block_types[i].endswith('-h'):
            indentation = ' '*8
        else:
            indentation = ' '*4
        code_blocks[i] = indent_lines(code_blocks[i], format,
                                      indentation)

    # After transforming align environments to separate equations
    # the problem with math labels in multiple eqs has disappeared.
    # (doconce.py applies align2equations, which takes all align
    # envirs and translates them to separate equations, but align*
    # environments are allowed.
    # Any output of labels in align means an error in the
    # align -> equation transformation...)
    math_labels = []
    multiple_math_labels = []  # sphinx has problems with multiple math labels
    for i in range(len(tex_blocks)):
        tex_blocks[i] = indent_lines(tex_blocks[i], format)
        # extract all \label{}s inside tex blocks and typeset them
        # with :label: tags
        label_regex = fix_latex( r'label\{(.+?)\}', application='match')
        labels = re.findall(label_regex, tex_blocks[i])
        if len(labels) == 1:
            tex_blocks[i] = '   :label: %s\n' % labels[0] + tex_blocks[i]
        elif len(labels) > 1:
            multiple_math_labels.append(labels)
        if len(labels) > 0:
            math_labels.extend(labels)
        tex_blocks[i] = re.sub(label_regex, '', tex_blocks[i])

        # fix latex constructions that do not work with sphinx math
        # (just remove them)
        commands = [r'\begin{equation}',
                    r'\end{equation}',
                    r'\begin{equation*}',
                    r'\end{equation*}',
                    #r'\begin{eqnarray}',
                    #r'\end{eqnarray}',
                    #r'\begin{eqnarray*}',
                    #r'\end{eqnarray*}',
                    #r'\begin{align}',
                    #r'\end{align}',
                    #r'\begin{align*}',
                    #r'\end{align*}',
                    r'\begin{multline}',
                    r'\end{multline}',
                    r'\begin{multline*}',
                    r'\end{multline*}',
                    #r'\begin{split}',
                    #r'\end{split}',
                    #r'\begin{gather}',
                    #r'\end{gather}',
                    #r'\begin{gather*}',
                    #r'\end{gather*}',
                    r'\[',
                    r'\]',
                    # some common abbreviations (newcommands):
                    r'\beqan',
                    r'\eeqan',
                    r'\beqa',
                    r'\eeqa',
                    r'\balnn',
                    r'\ealnn',
                    r'\baln',
                    r'\ealn',
                    r'\beq',
                    r'\eeq',  # the simplest name, contained in others, must come last!
                    ]
        for command in commands:
            tex_blocks[i] = tex_blocks[i].replace(command, '')
        # &=& -> &=
        tex_blocks[i] = re.sub('&\s*=\s*&', ' &= ', tex_blocks[i])
        # provide warnings for problematic environments

    # Replace all references to equations that have labels in math environments:
    for label in math_labels:
        filestr = filestr.replace('(:ref:`%s`)' % label, ':eq:`%s`' % label)

    multiple_math_labels_with_refs = [] # collect the labels with references
    for labels in multiple_math_labels:
        for label in labels:
            ref = ':eq:`%s`' % label  # ref{} is translated to eq:``
            if ref in filestr:
                multiple_math_labels_with_refs.append(label)

    if multiple_math_labels_with_refs:
        errwarn("""
*** warning: detected non-align math environment with multiple labels
    (Sphinx cannot handle this equation system - labels will be removed
    and references to them will be empty):""")
        for label in multiple_math_labels_with_refs:
            errwarn('    label{%s}' % label)
        print()

    filestr = insert_code_and_tex(filestr, code_blocks, tex_blocks, 'sphinx')

    # Remove all !bc ipy and !bc pyshell since interactive sessions
    # are automatically handled by sphinx without indentation
    # (just a blank line before and after)
    filestr = re.sub(r'^!bc +d?ipy *\n(.*?)^!ec *\n',
                     '\n\g<1>\n', filestr, re.DOTALL|re.MULTILINE)
    filestr = re.sub(r'^!bc +d?pyshell *\n(.*?)^!ec *\n',
                     '\n\g<1>\n', filestr, re.DOTALL|re.MULTILINE)

    # Check if we have custom pygments lexers
    if 'ipy' in code_block_types:
        if not has_custom_pygments_lexer('ipy'):
            envir2pygments['ipy'] = 'python'
    if 'do' in code_block_types:
        if not has_custom_pygments_lexer('doconce'):
            envir2pygments['do'] = 'text'

    # Make correct code-block:: language constructions
    legal_pygments_languages = get_legal_pygments_lexers()
    for key in set(code_block_types):
        if key in envir2pygments:
            if not envir2pygments[key] in legal_pygments_languages:
                errwarn("""*** warning: %s is not a legal Pygments language (lexer)
found in line:
  %s

    The 'text' lexer will be used instead.
""" % (envir2pygments[key], defs_line))
                envir2pygments[key] = 'text'

        #filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
        #                 '\n.. code-block:: %s\n\n' % envir2pygments[key], filestr,
        #                 flags=re.MULTILINE)

        # Check that we have code installed to handle pyscpro
        if 'pyscpro' in filestr and key == 'pyscpro':
            try:
                import icsecontrib.sagecellserver
            except ImportError:
                errwarn("""
*** warning: pyscpro for computer code (sage cells) is requested, but'
    icsecontrib.sagecellserver from https://github.com/kriskda/sphinx-sagecell
    is not installed. Using plain Python typesetting instead.""")
                key = 'pypro'

        if key == 'pyoptpro':
            if option('runestone'):
                filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
                    '\n.. codelens:: codelens_\n   :showoutput:\n\n',
                    filestr, flags=re.MULTILINE)
            else:
                filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
                                 '\n.. raw:: html\n\n',
                                 filestr, flags=re.MULTILINE)
        elif key == 'pyscpro':
            if option('runestone'):
                filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
                                 """
.. activecode:: activecode_
   :language: python

""", filestr, flags=re.MULTILINE)
            else:
                filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
                                 '\n.. sagecellserver::\n\n',
                                 filestr, flags=re.MULTILINE)
        elif key == 'pysccod':
            if option('runestone'):
                # Include (i.e., run) all previous code segments...
                # NOTE: this is most likely not what we want
                include = ', '.join([i for i in range(1, activecode_counter)])
                filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
                                 """
.. activecode:: activecode_
   :language: python
   "include: %s
""" % include, filestr, flags=re.MULTILINE)
            else:
                errwarn('*** error: pysccod for sphinx is not supported without the --runestone flag\n    (but pyscpro is via Sage Cell Server)')
                _abort()

        elif key == '':
            # any !bc with/without argument becomes a text block:
            filestr = re.sub(r'^!bc$', '\n.. code-block:: text\n\n', filestr,
                             flags=re.MULTILINE)
        elif key.endswith('hid'):
            if key in ('pyhid', 'jshid', 'htmlhid') and option('runestone'):
                # Allow runestone books to run hidden code blocks
                # (replace pyhid by pycod, then remove all !bc *hid)
                for i in range(len(code_block_types)):
                    if code_block_types[i] == key:
                        code_block_types[i] = key.replace('hid', 'cod')

                key2language = dict(py='python', js='javascript', html='html')
                language = key2language[key.replace('hid', '')]
                include = ', '.join([i for i in range(1, activecode_counter)])
                filestr = re.sub(r'^!bc +%s\s*\n' % key,
                                 """
.. activecode:: activecode_
   :language: %s
   :include: %s
   :hidecode:

""" % (language, include), filestr, flags=re.MULTILINE)
            else:
                # Remove hidden code block
                pattern = r'^!bc +%s\n.+?^!ec' % key
                filestr = re.sub(pattern, '', filestr,
                                 flags=re.MULTILINE|re.DOTALL)
        else:
            show_hide = False
            if key.endswith('-h'):
                key_orig = key
                key = key[:-2]
                show_hide = True
            # Use the standard sphinx code-block directive
            if key in envir2pygments:
                pygments_language = envir2pygments[key]
            elif key in legal_pygments_languages:
                pygments_language = key
            else:
                errwarn('*** error: detected code environment "%s"' % key)
                errwarn('    which is not registered in sphinx.py (sphinx_code)')
                errwarn('    or not a language registered in pygments')
                _abort()
            if show_hide:
                filestr = re.sub(r'^!bc +%s\s*\n' % key_orig,
                                 '\n.. container:: toggle\n\n    .. container:: header\n\n        **Show/Hide Code**\n\n    .. code-block:: %s\n\n' % \
                                 pygments_language, filestr, flags=re.MULTILINE)
                # Must add 4 indent in corresponding code_blocks[i], done above
            else:
                filestr = re.sub(r'^!bc +%s\s*\n' % key,
                                 '\n.. code-block:: %s\n\n' % \
                                 pygments_language, filestr, flags=re.MULTILINE)

    # any !bc with/without argument becomes a text block:
    filestr = re.sub(r'^!bc.*$', '\n.. code-block:: text\n\n', filestr,
                     flags=re.MULTILINE)
    filestr = re.sub(r'^!ec *\n', '\n', filestr, flags=re.MULTILINE)
    #filestr = re.sub(r'^!ec\n', '\n', filestr, flags=re.MULTILINE)
    #filestr = re.sub(r'^!ec\n', '', filestr, flags=re.MULTILINE)

    filestr = re.sub(r'^!bt *\n', '\n.. math::\n', filestr, flags=re.MULTILINE)
    filestr = re.sub(r'^!et *\n', '\n', filestr, flags=re.MULTILINE)
    # Fix lacking blank line after :label:
    filestr = re.sub(r'^(   :label: .+?)(\n *[^ ]+)', r'\g<1>\n\n\g<2>',
                     filestr, flags=re.MULTILINE)

    # Insert counters for runestone blocks
    if option('runestone'):
        codelens_counter = 0
        activecode_counter = 0
        lines = filestr.splitlines()
        for i in range(len(lines)):
            if '.. codelens:: codelens_' in lines[i]:
                codelens_counter += 1
                lines[i] = lines[i].replace('codelens_', 'codelens_%d' %
                                            codelens_counter)
            if '.. activecode:: activecode_' in lines[i]:
                activecode_counter += 1
                lines[i] = lines[i].replace('activecode_', 'activecode_%d' %
                                            activecode_counter)
        filestr = '\n'.join(lines)


    # Final fixes

    filestr = fix_underlines_in_headings(filestr)
    # Ensure blank line before and after comments
    filestr = re.sub(r'([.:;?!])\n^\.\. ', r'\g<1>\n\n.. ',
                     filestr, flags=re.MULTILINE)
    filestr = re.sub(r'(^\.\. .+)\n([^ \n]+)', r'\g<1>\n\n\g<2>',
                     filestr, flags=re.MULTILINE)

    # Line breaks interfer with tables and needs a final blank line too
    lines = filestr.splitlines()
    inside_block = False
    for i in range(len(lines)):
        if lines[i].startswith('<linebreakpipe>') and not inside_block:
            inside_block = True
            lines[i] = lines[i].replace('<linebreakpipe> ', '') + '\n'
            continue
        if lines[i].startswith('<linebreakpipe>') and inside_block:
            lines[i] = '|' + lines[i].replace('<linebreakpipe>', '')
            continue
        if inside_block and not lines[i].startswith('<linebreakpipe>'):
            inside_block = False
            lines[i] = '| ' + lines[i] + '\n'
    filestr = '\n'.join(lines)

    # Remove double !split (TOC with a prefix !split gives two !splits)
    pattern = '^.. !split\s+.. !split'
    filestr = re.sub(pattern, '.. !split', filestr, flags=re.MULTILINE)

    if option('html_links_in_new_window'):
        # Insert a comment to be recognized by automake_sphinx.py such that it
        # can replace the default links by proper modified target= option.
        #filestr = '\n\n.. NOTE: Open external links in new windows.\n\n' + filestr
        # Use JavaScript instead
        filestr = """.. raw:: html

        <script type="text/javascript">
        $(document).ready(function() {
            $("a[href^='http']").attr('target','_blank');
        });
        </script>

""" + filestr


    # Remove too much vertical space
    filestr = re.sub(r'\n{3,}', '\n\n', filestr)

    return filestr

def sphinx_ref_and_label(section_label2title, format, filestr):
    # Special fix early in the process:
    # Deal with !split - by default we place splits before
    # the all the topmost sections
    # (This must be done before labels are put above section
    # headings)
    if '!split' in filestr and not option('sphinx_keep_splits'):
        errwarn('*** warning: new !split inserted (override all existing !split)')
        # Note: the title is at this stage translated to a chapter heading!
        # This title/heading must be removed for the algorithm below to work
        # (remove it, then insert afterwards)
        pattern = r'^.. Document title:\n\n={3,9}.+?={3,9}'
        m = re.search(pattern, filestr, flags=re.MULTILINE)
        title_replacement = '<<<<<<<DOCUMENT TITLE>>>>>>>>>>>>' # "unlikely" str
        if m:
            title = m.group()
            filestr = filestr.replace(title, title_replacement)
        else:
            title = ''

        topmost_section = 0
        for i in [9, 7, 5]:
            if re.search(r'^%s' % ('='*i), filestr, flags=re.MULTILINE):
                topmost_section = i
                errwarn('    before every %s heading %s' % \
                        ('='*topmost_section, '='*topmost_section))
                errwarn('    because this strategy gives a well-functioning')
                errwarn('    table of contents in Sphinx')
                errwarn('    (use --sphinx_keep_splits to enforce your own !split commands)')
                break
        if topmost_section:
            # First remove all !split
            filestr = re.sub(r'^!split *\n', '', filestr, flags=re.MULTILINE)
            # Insert new splits before all topmost sections
            pattern = r'^%s (.+?) %s' % \
                      ('='*topmost_section, '='*topmost_section)
            lines = filestr.splitlines()
            for i in range(len(lines)):
                if re.search(pattern, lines[i]):
                    lines[i] = '!split\n' + lines[i]

            filestr = '\n'.join(lines)
        filestr = filestr.replace(title_replacement, title)

    filestr = ref_and_label_commoncode(section_label2title, format, filestr)

    # replace all references to sections:
    for label in section_label2title:
        filestr = filestr.replace('ref{%s}' % label, ':ref:`%s`' % label)

    # Not of interest after sphinx got equation references:
    #from common import ref2equations
    #filestr = ref2equations(filestr)

    # Replace remaining ref{x} as :ref:`x`
    filestr = re.sub(r'ref\{(.+?)\}', ':ref:`\g<1>`', filestr)

    return filestr

def sphinx_index_bib(filestr, index, citations, pubfile, pubdata):
    # allow user to force the use of original bibliography keys instead of numbered labels
    numbering = not option('sphinx_preserve_bib_keys', False)

    filestr = rst_bib(filestr, citations, pubfile, pubdata, numbering=numbering)
    from .common import INLINE_TAGS

    for word in index:
        # Drop verbatim, emphasize, bold, and math in index
        word2 = word.replace('`', '')
        word2 = word2.replace('$', '').replace('\\', '')
        word2 = re.sub(INLINE_TAGS['bold'],
                       r'\g<begin>\g<subst>\g<end>', word2,
                       flags=re.MULTILINE)
        word2 = re.sub(INLINE_TAGS['emphasize'],
                       r'\g<begin>\g<subst>\g<end>', word2,
                       flags=re.MULTILINE)

        # Typeset idx{word} as ..index::
        if '!' not in word and ',' not in word:
            # .. index:: keyword
            filestr = filestr.replace(
                'idx{%s}' % word,
                '\n.. index:: ' + word2 + '\n')
        elif '!' not in word:
            # .. index::
            #    single: keyword with comma
            filestr = filestr.replace(
                'idx{%s}' % word,
                '\n.. index::\n   single: ' + word2 + '\n')
        else:
            # .. index::
            #    single: keyword; subentry
            word3 = word2.replace('!', '; ')
            filestr = filestr.replace(
                'idx{%s}' % word,
                '\n.. index::\n   single: ' + word3 + '\n')

            # Symmetric keyword; subentry and subentry; keyword
            #filestr = filestr.replace(
            #    'idx{%s}' % word,
            #    '\n.. index::\n   pair: ' + word3 + '\n')
    return filestr

def sphinx_inline_comment(m):
    # Explicit HTML typesetting does not work, we just use bold
    name = m.group('name').strip()
    comment = m.group('comment').strip()

    global edit_markup_warning
    if (not edit_markup_warning) and \
           (name[:3] in ('add', 'del', 'edi') or '->' in comment):
        errwarn('*** warning: sphinx/rst is a suboptimal format for')
        errwarn('    typesetting edit markup such as')
        errwarn('    ' + m.group())
        errwarn('    Use HTML or LaTeX output instead, implement the')
        errwarn('    edits (doconce apply_edit_comments) and then use sphinx.')
        edit_markup_warning = True

    chars = {',': 'comma', ';': 'semicolon', '.': 'period'}
    if name[:4] == 'del ':
        for char in chars:
            if comment == char:
                return r' (**edit %s**: delete %s)' % (name[4:], chars[char])
        return r'(**edit %s**: **delete** %s)' % (name[4:], comment)
    elif name[:4] == 'add ':
        for char in chars:
            if comment == char:
                return r'%s (**edit %s: add %s**)' % (comment, name[4:], chars[char])
        return r' (**edit %s: add**) %s (**end add**)' % (name[4:], comment)
    else:
        # Ordinary name
        comment = ' '.join(comment.splitlines()) # '\s->\s' -> ' -> '
        if ' -> ' in comment:
            # Replacement
            if comment.count(' -> ') != 1:
                errwarn('*** wrong syntax in inline comment:')
                errwarn(comment)
                errwarn('(more than two ->)')
                _abort()
            orig, new = comment.split(' -> ')
            return r'(**%s: remove** %s) (**insert:**)%s (**end insert**)' % (name, orig, new)
        else:
            # Ordinary comment
            return r'[**%s**: %s]' % (name, comment)

def define(FILENAME_EXTENSION,
           BLANKLINE,
           INLINE_TAGS_SUBST,
           CODE,
           LIST,
           ARGLIST,
           TABLE,
           EXERCISE,
           FIGURE_EXT,
           CROSS_REFS,
           INDEX_BIB,
           TOC,
           ENVIRS,
           QUIZ,
           INTRO,
           OUTRO,
           filestr):
    if not 'rst' in BLANKLINE:
        # rst.define is not yet ran on these dictionaries, do it:
        from . import rst
        rst.define(FILENAME_EXTENSION,
                   BLANKLINE,
                   INLINE_TAGS_SUBST,
                   CODE,
                   LIST,
                   ARGLIST,
                   TABLE,
                   FIGURE_EXT,
                   INTRO,
                   OUTRO,
                   filestr)

    FILENAME_EXTENSION['sphinx'] = FILENAME_EXTENSION['rst']
    BLANKLINE['sphinx'] = BLANKLINE['rst']
    CODE['sphinx'] = CODE['rst']
    LIST['sphinx'] = LIST['rst']
    FIGURE_EXT['sphinx'] = {
        'search': ('.png', '.gif', '.jpg', '.jpeg'),
        'convert': ('.png', '.gif', '.jpg')}
    CROSS_REFS['sphinx'] = sphinx_ref_and_label
    INDEX_BIB['sphinx'] = sphinx_index_bib
    TABLE['sphinx'] = TABLE['rst']
    EXERCISE['sphinx'] = EXERCISE['rst']
    ENVIRS['sphinx'] = ENVIRS['rst']
    INTRO['sphinx'] = INTRO['rst'].replace(
        '.. Automatically generated reStructuredText',
        '.. Automatically generated Sphinx-extended reStructuredText')

    # make true copy of INLINE_TAGS_SUBST:
    INLINE_TAGS_SUBST['sphinx'] = {}
    for tag in INLINE_TAGS_SUBST['rst']:
        INLINE_TAGS_SUBST['sphinx'][tag] = INLINE_TAGS_SUBST['rst'][tag]

    # modify some tags:
    #INLINE_TAGS_SUBST['sphinx']['math'] = r'\g<begin>:math:`\g<subst>`\g<end>'
    # Important to strip the math expression
    INLINE_TAGS_SUBST['sphinx']['math'] = lambda m: r'%s:math:`%s`%s' % (m.group('begin'), m.group('subst').strip(), m.group('end'))
    #INLINE_TAGS_SUBST['sphinx']['math2'] = r'\g<begin>:math:`\g<latexmath>`\g<end>'
    INLINE_TAGS_SUBST['sphinx']['math2'] = lambda m: r'%s:math:`%s`%s' % (m.group('begin'), m.group('latexmath').strip(), m.group('end'))
    INLINE_TAGS_SUBST['sphinx']['figure'] = sphinx_figure
    INLINE_TAGS_SUBST['sphinx']['movie'] = sphinx_movie
    INLINE_TAGS_SUBST['sphinx']['inlinecomment'] = sphinx_inline_comment
    CODE['sphinx'] = sphinx_code  # function for typesetting code

    ARGLIST['sphinx'] = {
        'parameter': ':param',
        'keyword': ':keyword',
        'return': ':return',
        'instance variable': ':ivar',
        'class variable': ':cvar',
        'module variable': ':var',
        }

    TOC['sphinx'] = lambda s, f: ''  # Sphinx automatically generates a toc
    QUIZ['sphinx'] = sphinx_quiz



#---------------------------------------------------------------------------
def sphinx_code_orig(filestr, format):
    # NOTE: THIS FUNCTION IS NOT USED!!!!!!

    # In rst syntax, code blocks are typeset with :: (verbatim)
    # followed by intended blocks. This function indents everything
    # inside code (or TeX) blocks.

    # grab #sphinx code-blocks: cod=python cpp=c++ etc line
    # (do this before code is inserted in case verbatim blocks contain
    # such specifications for illustration)
    m = re.search(r'#\s*[Ss]phinx\s+code-blocks?:(.+?)\n', filestr)
    if m:
        defs_line = m.group(1)
        # turn defs into a dictionary definition:
        defs = {}
        for definition in defs_line.split():
            key, value = definition.split('=')
            defs[key] = value
    else:
        # default mappings:
        defs = dict(cod='python',
                    pro='python',
                    pycod='python', cycod='cython',
                    pypro='python', cypro='cython',
                    fcod='fortran', fpro='fortran',
                    ccod='c', cppcod='c++',
                    cpro='c', cpppro='c++',
                    mcod='matlab', mpro='matlab',
                    plcod='perl', plpro='perl',
                    shcod='bash', shpro='bash',
                    rbcod='ruby', rbpro='ruby',
                    sys='console',
                    dat='python',
                    ipy='python',
                    xmlcod='xml', xmlpro='xml', xml='xml',
                    htmlcod='html', htmlpro='html', html='html',
                    texcod='latex', texpro='latex', tex='latex',
                    )
        # (the "python" typesetting is neutral if the text
        # does not parse as python)

    # first indent all code/tex blocks by 1) extracting all blocks,
    # 2) intending each block, and 3) inserting the blocks:
    filestr, code_blocks, tex_blocks = remove_code_and_tex(filestr, format)
    for i in range(len(code_blocks)):
        code_blocks[i] = indent_lines(code_blocks[i], format)
    for i in range(len(tex_blocks)):
        tex_blocks[i] = indent_lines(tex_blocks[i], format)
        # remove all \label{}s inside tex blocks:
        tex_blocks[i] = re.sub(fix_latex(r'\label\{.+?\}', application='match'),
                              '', tex_blocks[i])
        # remove those without \ if there are any:
        tex_blocks[i] = re.sub(r'label\{.+?\}', '', tex_blocks[i])
        # side effects: `label{eq1}` as verbatim, but this is mostly a
        # problem for doconce documentation and can be rephrased...

        # fix latex constructions that do not work with sphinx math
        commands = [r'\begin{equation}',
                    r'\end{equation}',
                    r'\begin{equation*}',
                    r'\end{equation*}',
                    r'\begin{eqnarray}',
                    r'\end{eqnarray}',
                    r'\begin{eqnarray*}',
                    r'\end{eqnarray*}',
                    r'\begin{align}',
                    r'\end{align}',
                    r'\begin{align*}',
                    r'\end{align*}',
                    r'\begin{multline}',
                    r'\end{multline}',
                    r'\begin{multline*}',
                    r'\end{multline*}',
                    r'\begin{split}',
                    r'\end{split}',
                    r'\begin{gather}',
                    r'\end{gather}',
                    r'\begin{gather*}',
                    r'\end{gather*}',
                    r'\[',
                    r'\]',
                    # some common abbreviations (newcommands):
                    r'\beqan',
                    r'\eeqan',
                    r'\beqa',
                    r'\eeqa',
                    r'\balnn',
                    r'\ealnn',
                    r'\baln',
                    r'\ealn',
                    r'\beq',
                    r'\eeq',  # the simplest, contained in others, must come last!
                    ]
        for command in commands:
            tex_blocks[i] = tex_blocks[i].replace(command, '')
        tex_blocks[i] = re.sub('&\s*=\s*&', ' &= ', tex_blocks[i])
        # provide warnings for problematic environments
        #if '{alignat' in tex_blocks[i]:
        #    errwarn('*** warning: the "alignat" environment will give errors in Sphinx:\n' + tex_blocks[i] + '\n')


    filestr = insert_code_and_tex(filestr, code_blocks, tex_blocks, 'rst')

    for key in defs:
        language = defs[key]
        if not language in legal_pygments_languages:
            raise TypeError('%s is not a legal Pygments language '\
                            '(lexer) in line with:\n  %s' % \
                                (language, defs_line))
        #filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
        #                 '\n.. code-block:: %s\n\n' % defs[key], filestr,
        #                 flags=re.MULTILINE)
        cpattern = re.compile(r'^!bc\s+%s\s*\n' % key, flags=re.MULTILINE)
        filestr, n = cpattern.subn('\n.. code-block:: %s\n\n' % defs[key], filestr)
        errwarn(key + ' ' + n)
        if n > 0:
            errwarn('sphinx: %d subst %s by %s' % (n, key, defs[key]))

    # any !bc with/without argument becomes a py (python) block:
    #filestr = re.sub(r'^!bc.+\n', '\n.. code-block:: py\n\n', filestr,
    #                 flags=re.MULTILINE)
    cpattern = re.compile(r'^!bc.+$', flags=re.MULTILINE)
    filestr = cpattern.sub('\n.. code-block:: py\n\n', filestr)

    filestr = re.sub(r'^!ec *\n', '\n', filestr, flags=re.MULTILINE)
    #filestr = re.sub(r'^!ec\n', '\n', filestr, flags=re.MULTILINE)
    #filestr = re.sub(r'^!ec\n', '', filestr, flags=re.MULTILINE)
    filestr = re.sub(r'^!bt *\n', '\n.. math::\n\n', filestr,
                     flags=re.MULTILINE)
    filestr = re.sub(r'^!et *\n', '\n\n', filestr,
                     flags=re.MULTILINE)

    return filestr

def sphinx_code_newmathlabels(filestr, format):
    # NOTE: THIS FUNCTION IS NOT USED!!!!!!

    # In rst syntax, code blocks are typeset with :: (verbatim)
    # followed by intended blocks. This function indents everything
    # inside code (or TeX) blocks.

    # grab #sphinx code-blocks: cod=python cpp=c++ etc line
    # (do this before code is inserted in case verbatim blocks contain
    # such specifications for illustration)
    m = re.search(r'#\s*[Ss]phinx\s+code-blocks?:(.+?)\n', filestr)
    if m:
        defs_line = m.group(1)
        # turn defs into a dictionary definition:
        defs = {}
        for definition in defs_line.split():
            key, value = definition.split('=')
            defs[key] = value
    else:
        # default mappings:
        defs = dict(cod='python', pycod='python', cppcod='c++',
                    fcod='fortran', ccod='c',
                    pro='python', pypro='python', cpppro='c++',
                    fpro='fortran', cpro='c',
                    sys='console', dat='python')
        # (the "python" typesetting is neutral if the text
        # does not parse as python)

    # First indent all code/tex blocks by 1) extracting all blocks,
    # 2) intending each block, and 3) inserting the blocks.
    # In between, handle the math blocks.

    filestr, code_blocks, tex_blocks = remove_code_and_tex(filestr, format)
    for i in range(len(code_blocks)):
        code_blocks[i] = indent_lines(code_blocks[i], format)

    math_labels = []
    for i in range(len(tex_blocks)):
        tex_blocks[i] = indent_lines(tex_blocks[i], format)
        # extract all \label{}s inside tex blocks and typeset them
        # with :label: tags
        label_regex1 = fix_latex(r'\label\{(.+?)\}', application='match')
        label_regex2 = fix_latex( r'label\{(.+?)\}', application='match')
        math_labels.extend(re.findall(label_regex1, tex_blocks[i]))
        tex_blocks[i] = re.sub(label_regex1,
                              r' :label: \g<1> ', tex_blocks[i])
        # handle also those without \ if there are any:
        math_labels.extend(re.findall(label_regex2, tex_blocks[i]))
        tex_blocks[i] = re.sub(label_regex2, r' :label: \g<1> ', tex_blocks[i])

    # replace all references to equations:
    for label in math_labels:
        filestr = filestr.replace(':ref:`%s`' % label, ':eq:`%s`' % label)

    filestr = insert_code_and_tex(filestr, code_blocks, tex_blocks, 'rst')

    for key in defs:
        language = defs[key]
        if not language in legal_pygments_languages:
            raise TypeError('%s is not a legal Pygments language '\
                            '(lexer) in line with:\n  %s' % \
                                (language, defs_line))
        #filestr = re.sub(r'^!bc\s+%s\s*\n' % key,
        #                 '\n.. code-block:: %s\n\n' % defs[key], filestr,
        #                 flags=re.MULTILINE)
        cpattern = re.compile(r'^!bc\s+%s\s*\n' % key, flags=re.MULTILINE)
        filestr = cpattern.sub('\n.. code-block:: %s\n\n' % defs[key], filestr)

    # any !bc with/without argument becomes a py (python) block:
    #filestr = re.sub(r'^!bc.+\n', '\n.. code-block:: py\n\n', filestr,
    #                 flags=re.MULTILINE)
    cpattern = re.compile(r'^!bc.+$', flags=re.MULTILINE)
    filestr = cpattern.sub('\n.. code-block:: py\n\n', filestr)

    filestr = re.sub(r'!ec *\n', '\n', filestr)
    #filestr = re.sub(r'!ec\n', '\n', filestr)
    #filestr = re.sub(r'!ec\n', '', filestr)
    filestr = re.sub(r'!bt *\n', '\n.. math::\n   :nowrap:\n\n', filestr)
    filestr = re.sub(r'!et *\n', '\n\n', filestr)

    return filestr


#ALE
# sphinx_dir must be defined in bin/doconce because it imports sphinx,
# and in misc.py this is interpreted as the local doconce sphinx.py
# file...

def _usage_sphinx_dir():
    print("Usage: doconce sphinx_dir short-title=\"some short title\" version=0.1 theme=themename dirname=sphinx-rootdir "
          "toc_depth=2 intersphinx  doconcefile [--runestone]\n\n"
          "Additional command-line arguments: theme_dir=directoryname conf.py=filename logo=filename favicon=filename\n\n"
          "The sphinx_dir command creates a directory (dirname) for compiling a sphinx version of a doconce document. The option --runestone "
          "makes a directory for RunestoneInteractive documents (slightly different from a standard sphinx directory).\n\n"
          "The default dirname is \"sphinx-rootdir\". The intersphinx option is used for links to other sphinx documents.\n\n"
          "toc_depth governs the depth in the table of contents, default value is 2 (sections and subsections are included).\n\n"
          "The copyright in the footer of Sphinx documents is based on the AUTHOR field(s) in doconcefile, and if no "
          "{copyright...} specification is given in the field(s), all authors are listed.\n\n"
          "The theme option can be chosen from basic sphinx themes (agogo, basic, bizstyle, classic, default, epub, haiku, "
          "nature, pyramid, scrolls, sphinxdoc, traditional), extra themes that can be installed (alabaster, bootstrap, "
          "cloud, redcloud, solarized, sphinx_rtd_theme, 'draft', 'flat', 'modern5', 'minimal5', 'responsive'),or themes "
          "that come with DocOnce (ADCtheme, agni, basicstrap, bloodish, cbc, fenics, "
          "fenics_classic, fenics_minimal1, fenics_minimal2, jal, pylons, scipy_lectures, slim-agogo, uio, uio2, vlinux-theme). "
          "See bundled/sphinx_themes/install.txt for how to install extra themes.\n\n"
          "A custom theme can also be specified by giving a \"_themes-like\" directory: theme_dir=dirname, where dirname "
          "can be any path (DocOnce will copy this directory to the sphinx root directory). Note that this must be the "
          "*theme directory* with files like layout.html and theme.conf (not the entire Python package for a theme.) "
          "Alternatively, one can use a standard theme, but provide one's own tailored conf.py file through the conf.py=filename option.\n" 
          "If the file (filename) has empty strings for the project and copyright, those values as well as the variables"
          " depending on these values will get content based on the title, author, and copyright from the DocOnce source"
          " file mydoc.do.txt. (It is recommended to leave project and copyright as empty strings, and latex_documents, "
          "texinfo_documents, and man_pages as empty lists.)\n\n---\n\n"
          "The steps for copying files to the sphinx directory and compiling the document is done by an automatically generated script:\n\n"
          "  python automake_sphinx.py [doconce format options]\n\n"
          "The master file can be split into parts. Here is the typical code:\n\n"
          "  doconce format sphinx mydoc.do.txt\n"
          "  doconce split_rst mydoc\n"
          "  doconce sphinx_dir mydoc  # or with a lot of options...\n"
          "  python automake_sphinx.py\n\n"
          "Note that split_rst must be run prior to sphinx_dir!")

def sphinx_dir():
    if len(sys.argv) < 2:
        _usage_sphinx_dir()
        sys.exit(1)

    # Grab title, author/copyright, version from the command line
    # (set default values first)
    short_title = None
    version = '1.0'
    theme = 'classic'
    doconce_files = []
    sphinx_rootdir = 'sphinx-rootdir'
    toc_depth = 2
    conf_py = None
    theme_dir = None
    logo = None
    favicon = None
    intersphinx = False
    runestone = False
    # For backward compatibility we need to detect title, copyright and author
    # on the command line
    title_cml = copyright_cml = author_cml = None

    # Get user's info
    for arg in sys.argv[1:]:
        if arg.startswith('short_title='):
            short_title = arg.split('=')[1]
        elif arg.startswith('version='):
            version = arg.split('=')[1]
        elif arg.startswith('dirname='):
            sphinx_rootdir = arg.split('=')[1]
        elif arg.startswith('toc_depth='):
            toc_depth = int(arg.split('=')[1])
        elif arg.startswith('theme='):
            theme = arg.split('=')[1]
        elif arg.startswith('conf.py='):
            conf_py = arg.split('=')[1]
        elif arg.startswith('theme_dir='):
            theme_dir = arg.split('=')[1]
        elif arg.startswith('logo='):
            logo = arg.split('=')[1]
        elif arg.startswith('favicon='):
            favicon = arg.split('=')[1]
        elif arg.startswith('intersphinx'):
            intersphinx = True
        elif arg == '--runestone':
            runestone = True
        elif arg.startswith('author='):
            author_cml = arg.split('=')[1]
        elif arg.startswith('copyright='):
            copyright_cml = arg.split('=')[1]
        elif arg.startswith('title='):
            title_cml = arg.split('=')[1]
        else:
            doconce_files.append(arg)

    if not doconce_files:
        print('must have (at least one) doconce file as argument')
        print('doconce sphinx_dir somefile.do.txt')
        sys.exit(1)
    try:
        import sphinx
    except ImportError:
        print('Unable to import sphinx. Install sphinx from sphinx.pocoo.org.')
        print('On Debian systems, install the \'python-sphinx\' package.')
        sys.exit(1)
    if float(sphinx.__version__[:3]) < 1.1:
        print('Abort: sphinx version >= 1.1 required')
        sys.exit(1)

    if len(doconce_files) > 1:
        print('can only specify one main doconce file')
        print('(here you have specified %s)' % ', '.join(doconce_files))
        sys.exit(1)

    filename = doconce_files[0]
    if filename.endswith('do.txt'):
        filename = filename[:-7]
    else:
        if not os.path.isfile(filename + '.do.txt'):
            print('%s.do.txt does not exist' % filename)
            sys.exit(1)

    parts = glob.glob('._%s[0-9][0-9][0-9].rst' % filename)
    parts = [part[:-4] for part in sorted(parts)]

    # Extract title, author from .do.txt file (try mako processed file first)
    fstr = load_preprocessed_doconce_file(filename)
    if 'TITLE:' in fstr:
        for line in fstr.splitlines():
            if line.startswith('TITLE:'):
                title = line[6:].strip()
                break # use the first TITLE spec
        if title_cml is not None:
            print('*** error: cannot give title= on the command line when TITLE is specified in the DocOnce file', filename)
            sys.exit(1)
    else:
        # No TITLE in the DocOnce file
        if title_cml is None:
            title = 'No title'
        else:
            title = title_cml

    if short_title is None:
        # Default is to use title
        short_title = title

    import doconce.doconce
    # The following can be misleading if there are examples on
    # various AUTHOR: in e.g. vertabim code in the .do.txt file
    authors_and_institutions, dummy1, dummy2, dummy3, dummy4, dummy5 = \
         doconce.doconce.interpret_authors(fstr, 'sphinx')
    if authors_and_institutions:
        author = [a for a, i, e in authors_and_institutions]

        if len(author) == 1:
            author = author[0]
        else:
            author = ', '.join(author[:-1]) + ' and ' + author[-1]
        if author_cml is not None:
            print('*** error: cannot give author= on the command line when AUTHOR is specified in the DocOnce file', filename)
            sys.exit(1)
    else:
        if author_cml is None:
            author = 'No author'
        else:
            author = author_cml

    from doconce.common import get_copyfile_info
    copyright_filename = '.' + filename + '.copyright'
    copyright_ = get_copyfile_info(copyright_filename=copyright_filename,
                                   format='sphinx')
    # copyright_ is None if no copyright and no doconce citation
    # copyright_ is 'Made with DocOnce' if just citation and no copyright,
    # otherwise it is a text with authors/institutions and optionally citation
    # in conf.py we *always* include a copyright with authors, so if
    # copyright is None or just 'Made with DocOnce' we add authors and year
    if isinstance(copyright_, str) and 'DocOnce' in copyright_:
        cite_doconce = True
        if copyright_ == 'Made with DocOnce':
            copyright_ = None  # No specified authors or institutions
    else:
        cite_doconce = False

    if copyright_ is None:
        # No copyright given in the file, use copyright= from command
        # line, if it was provided, or use this year and author
        if copyright_cml is None:
            # use this year, author
            this_year = time.asctime().split()[4]
            copyright_ = this_year + ', ' + author
            if cite_doconce:
                copyright_ += '. Made with DocOnce'
        else:
            copyright_ = copyright_cml
    else:
        if copyright_cml is not None:
            print('*** error: cannot give copyright= on the command line when {coppyright...} is specified as part of AUTHOR in the DocOnce file', filename)
            sys.exit(1)

    chapters = '=========' in fstr
    if chapters:
        toc_depth += 1

    if theme_dir is not None and conf_py is None:
        print('theme_dir is given, but then also conf_py must be specified')
        print('Abort!')
        sys.exit(1)

    print('title:', title)
    print('author:', author)
    print('copyright:', copyright_)
    print('theme:', theme)
    print()
    time.sleep(1.5)

    #make_conf_py_runestone(themes, theme, title, short_title,
    #                       copyright_, logo, favicon, intersphinx)

    if os.path.isdir(sphinx_rootdir):
        shutil.rmtree(sphinx_rootdir)
    if sys.platform.startswith('win'):
        f1 = open('tmp_sphinx_gen.bat', 'w')
        f2 = open('tmp_sphinx_gen.inp', 'w')
        f1.write("echo Making %(sphinx_rootdir)s\n"
                 "mkdir %(sphinx_rootdir)s\n"
                 "sphinx-quickstart < tmp_sphinx_gen.inp\n" % vars())
        f2.write(("%(sphinx_rootdir)s\n"
                  "n\n"
                  "_\n"
                  "%(title)s\n"
                  "%(author)s\n"
                  "%(version)s\n"
                  "%(version)s\n"
                  "en\n"
                  ".rst\n"
                  "index\n"
                  "y\n"
                  "y\n"
                  "n\n"
                  "n\n"
                  "n\n"
                  "n\n"
                  "n\n"
                  "y\n"
                  "n\n"
                  "y\n"
                  "y\n"
                  "y\n") % vars())
        f1.close()
        f2.close()
        system('tmp_sphinx_gen',
               failure_info='to generate sphinx directory')
        #os.remove('tmp_sphinx_gen.bat')
        #os.remove('tmp_sphinx_gen.inp')
    else:
        f = open('tmp_sphinx_gen.sh', 'w')
        f.write(("#!/bin/bash\n"
                 "echo Making %(sphinx_rootdir)s\n"
                 "mkdir %(sphinx_rootdir)s\n"
                 "sphinx-quickstart <<EOF\n"
                 "%(sphinx_rootdir)s\n"
                 "n\n"
                 "_\n"
                 "%(title)s\n"
                 "%(author)s\n"
                 "%(version)s\n"
                 "%(version)s\n"
                 "en\n"
                 ".rst\n"
                 "index\n"
                 "y\n"
                 "y\n"
                 "n\n"
                 "n\n"
                 "n\n"
                 "n\n"
                 "n\n"
                 "y\n"
                 "n\n"
                 "y\n"
                 "y\n"
                 "y\n\n"
                 "EOF") % vars())
        f.close()
        system('sh tmp_sphinx_gen.sh',
               failure_info='to generate sphinx directory')
        #os.remove('tmp_sphinx_gen.sh')
    os.chdir(sphinx_rootdir)

    sphinx_basic_themes = 'agogo basic bizstyle classic default epub haiku nature pyramid scrolls sphinxdoc traditional'.split()
    # See what is installed of other themes
    sphinx_potentially_installed_themes = 'alabaster bootstrap cloud redcloud solarized impressjs sphinx_rtd_theme dark ' \
                                          'flat modern5 minimal5 responsive'.split()
    sphinx_installed_themes = []
    try:
        import alabaster
        sphinx_installed_themes.append('alabaster')
    except ImportError:
        pass
    try:
        import sphinx_bootstrap_theme
        sphinx_installed_themes.append('bootstrap')
    except ImportError:
        pass
    try:
        import cloud_sptheme
        sphinx_installed_themes.append('cloud')
        sphinx_installed_themes.append('redcloud')
    except ImportError:
        pass
    try:
        import sphinxjp.themes.impressj
        sphinx_installed_themes.append('impressjs')
    except ImportError:
        pass
    try:
        import sphinxjp.themes.solarized
        sphinx_installed_themes.append('solarized')
    except ImportError:
        pass
    try:
        import sphinx_rtd_theme
        sphinx_installed_themes.append('sphinx_rtd_theme')
    except ImportError:
        pass
    try:
        import tinkerer
        import tinkerer.paths
        sphinx_installed_themes += ['boilerplate', 'dark', 'flat', 'modern5', 'minimal5', 'responsive']
    except ImportError:
        pass

    if theme_dir is None:
        # Copy themes
        import doconce.common
        install_dir = doconce.common.where()
        shutil.copy(os.path.join(install_dir, 'sphinx_themes.zip'), os.curdir)
        devnull = 'NUL' if sys.platform.startswith('win') else '/dev/null'
        system('unzip sphinx_themes.zip > %s' % devnull)
        os.remove('sphinx_themes.zip')
        os.rename('sphinx_themes', '_themes')
        files = sorted(glob.glob(os.path.join('_themes', '*')))
        themes = [name[8:] for name in files
                  if os.path.isdir(name)] + \
                  sphinx_basic_themes + sphinx_installed_themes

        if conf_py is None:
            print('These Sphinx themes were found:', ', '.join(sorted(themes)))
            make_conf_py(themes, theme, title, short_title, copyright_,
                         logo, favicon, intersphinx)
    else:
        # Copy theme_dir
        if not os.path.isdir('_themes'):
            os.mkdir('_themes')
        shutil.copy(theme_dir, os.path.join('_themes', theme_dir))


    if conf_py is not None:
        shutil.copy(os.path.join(os.pardir, conf_py), 'conf.py')

        # Edit the file appropriately if title, author, or copyright are given
        with open('conf.py', 'r') as f:
            conf_text = f.read()

        # Make conf.py ready for latex, man pages, texinfo, epub etc
        # with current doc info
        pattern = r'latex_documents = \[.*?\]'
        replacement = r"latex_documents = [('index', 'doc.tex', project, u'%s',  'manual'),]" % author
        conf_text = re.sub(pattern, replacement, conf_text,
                           flags=re.DOTALL)

        pattern = r'man_pages = \[.*?^# '
        replacement = r"man_pages = [('index', 'doc', project, [u'%s'], 1),]\n\n# " % author
        conf_text = re.sub(pattern, replacement, conf_text,
                           flags=re.DOTALL|re.MULTILINE)

        pattern = r'texinfo_documents = \[.*?\]'
        replacement = r"texinfo_documents = [('index', 'doc', project, u'%s', 'doc', 'One line description of project.', 'Miscellaneous'),]" % author
        conf_text = re.sub(pattern, replacement, conf_text,
                           flags=re.DOTALL)

        pattern = r'''project *= *u?['"](.*?)['"]'''
        m = re.search(pattern, conf_text)
        if m:
            if m.group(1).strip() == '':
                # No title found, take the one from .do.txt
                conf_text = re.sub(pattern, "project = u'%s'" % title,
                                   conf_text)
                conf_text = re.sub(r'html_title.+', "html_title = project", conf_text)
                conf_text = re.sub(r'epub_title.+', "epub_title = project", conf_text)
        pattern = '''^short_title *= *project'''
        m = re.search(pattern, conf_text, flags=re.MULTILINE)
        if short_title != title and m:
            # Update short_title
            conf_text = re.sub(pattern, "short_title = u'%s'" % short_title,
                               conf_text, flags=re.MULTILINE)

        # Fix copyright issues
        pattern = r'''copyright *= *u?['"](.*?)['"]'''
        m = re.search(pattern, conf_text)
        if m:
            if m.group(1).strip() == '':
                # No copyright found, take the one from .do.txt
                conf_text = re.sub(pattern, "copyright = u'%s'" % copyright_,
                                   conf_text)
                conf_text = re.sub(r'epub_copyright.+', "epub_copyright = u'%s'" % copyright_, conf_text)
                conf_text = re.sub(r'epub_author.+', "epub_author = u'%s'" % author, conf_text)
                conf_text = re.sub(r'epub_publisher.+', "epub_publisher = u'%s'" % author, conf_text)
        with open('conf.py', 'w') as f:
            f.write(conf_text)

    # Make a script that can generate all available sphinx themes
    f = open('make_themes.sh', 'w')
    f.write("#!/bin/sh\n"
            "# Make all themes given on the command line (or if no themes are\n"
            "# given, make all themes in _themes/)\n"
            "\n"
            "if [ $# -gt 0 ]; then\n"
            "    themes=$@\n"
            "else\n"
            "    themes=\"%s\"\n"
            "fi\n"
            "\n"
            "for theme in $themes; do\n"
            "    echo \"building theme $theme...\"\n"
            "    doconce subst -m \"^html_theme =.*$\" \"html_theme = '$theme'\" conf.py\n"
            "    make clean\n"
            "    make html\n"
            "    mv -f _build/html sphinx-$theme\n"
            "done\n"
            "echo\n"
            "echo \"Here are the built themes:\"\n"
            "ls -d sphinx-*\n"
            "echo \"for i in sphinx-*; do google-chrome $i/index.html; done\"\n"
            "\n" % (' '.join(themes)))
    f.close()
    os.chmod('make_themes.sh', 0o755)

    # Make index.rst main file for the document
    f = open('index.rst', 'w')
    if parts:
        files = parts
    else:
        files = [filename]
    filenames = '\n   '.join(files)
    title_underline = '='*len(title)
    dropdown_scrollbar = '' if not 'bootstrap' in theme else ("\n"
                                                              ".. raw:: html\n\n"
                                                              "  <style type=\"text/css\">\n"
                                                              "    .dropdown-menu {\n"
                                                              "      height: auto;\n"
                                                              "      max-height: 400px;\n"
                                                              "      overflow-x: hidden;\n"
                                                              "  }\n"
                                                              "  </style>\n")
    f.write((".. Master file automatically created by doconce sphinx_dir\n\n"
             "%(dropdown_scrollbar)s\n"
             "%(title)s\n"
             "%(title_underline)s\n\n"
             "Contents:\n\n"
             ".. toctree::\n"
             "   :maxdepth: %(toc_depth)s\n\n"
             "   %(filenames)s") % vars())

    # Do we have idx{} commands in the document? Cannot just check fstr,
    # that is the top/master file. Need to check the entire file.
    dotext = load_preprocessed_doconce_file(filename, dirpath=os.pardir)
    if 'idx{' in dotext:
        f.write(("\nIndex\n"
                 "=====\n\n"
                 "* :ref:`genindex`\n"))
    f.close()
    os.chdir(os.pardir)

    # Make the automake_sphinx.py compilation file for automatically
    # preparing the sphinx document, copying various files, and running make

    f = open('automake_sphinx.py', 'w')
    f.write(("#!/usr/bin/env python\n"
            "# -*- coding: utf-8 -*-\n"
            "# Autogenerated file (by doconce sphinx_dir).\n"
            "# Purpose: create HTML Sphinx version of document \"%(filename)s\".\n"
            "#\n"
            "# Note: doconce clean removes this file, so if you edit the file,\n"
            "# rename it to avoid automatic removal.\n\n"
            "# To force compilation of the doconce file to sphinx format, remove\n"
            "# the sphinx (.rst) file first.\n"
            "#\n"
            "# Command-line arguments are transferred to the \"doconce format sphinx\"\n"
            "# compilation command.\n"
            "#\n\n"
            "import glob, sys, os, subprocess, shutil, logging\n"
            "logging.basicConfig(\n"
            "    filename='automake_sphinx.log', filemode='w', level=logging.DEBUG,\n"
            "    format='%%(asctime)s - %%(levelname)s - %%(message)s',\n"
            "    datefmt='%%Y.%%m.%%d %%I:%%M:%%S %%p')\n\n\n"
            "command_line_options = ' '.join(['\"%%s\"' %% arg for arg in sys.argv[1:]])\n\n"
            "sphinx_rootdir = '%(sphinx_rootdir)s'\n"
            "source_dir = sphinx_rootdir\n\n"
            "if not os.path.isdir(sphinx_rootdir):\n"
            "    print(\"\"\"*** error: %%(sphinx_rootdir)s does not exist. This means unsuccessful\n"
            "    doconce sphinx_dir command. Try to upgrade to latest DocOnce version.\n"
            "    (The script tmp_sphinx_gen.sh runs sphinx-quickstart - it may have failed.)\n"
            "\"\"\" %% vars())\n"
            "    sys.exit(1)\n\n"
            "def system(cmd, capture_output=False, echo=True):\n"
            "    if echo:\n"
            "        print('running', cmd)\n"
            "    if capture_output:\n"
            "        failure, outtext = subprocess.getstatusoutput(cmd) # Unix/Linux only\n"
            "    else:\n"
            "        failure = os.system(cmd)\n"
            "    if failure:\n"
            "        print('Could not run', cmd)\n"
            "        logging.critical('Could not run %%s' %% cmd)\n"
            "        sys.exit(1)\n"
            "    if capture_output:\n"
            "        return outtext\n\n") % vars())
    if parts == []:
        f.write(("# Compile the doconce file if a sphinx version of it is not found\n"
                 "filename = '%(filename)s'\n"
                 "if not os.path.isfile(filename + '.rst'):\n"
                 "    # Transform doconce format to sphinx format and copy to sphinx directory\n"
                 "    cmd = 'doconce format sphinx %%s %%s' %% (filename, command_line_options)\n"
                 "    print(cmd)\n"
                 "    logging.info('running %%s' %% cmd)\n"
                 "    system(cmd)\n\n"
                 "    # Fix: utf-8 encoding for non-English chars\n"
                 "    cmd = 'doconce guess_encoding %%s.rst' %% filename\n"
                 "    enc = system(cmd, capture_output=True)\n"
                 "    if enc == \"iso-8859-1\":\n"
                 "        # sphinx does not like non-English characters in iso-8859-1\n"
                 "        cmd = 'doconce change_encoding iso-8859-1 utf-8 %%s.rst' %% filename\n"
                 "        logging.info('running %%s' %% cmd)\n"
                 "        system(cmd)\n\n"
                 "# Copy generated sphinx file to sphinx directory\n"
                 "logging.info('copy: %%s.rst to %%s and reading the content' %% (filename, source_dir))\n"
                 "shutil.copy('%%s.rst' %% filename, source_dir)\n"
                 "with open('%%s.rst' %% filename, 'r') as rst_file:\n"
                 "    rst_text = rst_file.read()\n") % vars())
    else:
        # user has run doconce split_rst so doconce format is already
        # run and all parts must be copied
        parts_names = str(parts)
        f.write(("# Copy generated sphinx files to sphinx root directory\n"
                 "filename = '%(filename)s'\n"
                 "rst_text = ''  # holds all text in all .rst files\n"
                 "for part in %(parts_names)s:\n"
                 "    shutil.copy('%%s.rst' %% part, source_dir)\n"
                 "    with open('%%s.rst' %% part, 'r') as rst_file:\n"
                 "        rst_text += rst_file.read()\n")
                % vars())

    f.write(("\n# Copy figures and movies directories\n"
             "figdirs = glob.glob('fig*') + glob.glob('mov*')\n"
             "for figdir in figdirs:\n"
             "    destdir = None\n"
             "    if figdir.startswith('fig'):\n"
             "        # Figures can be anywhere (copied by sphinx to _images)\n"
             "        destdir = os.path.join(source_dir, figdir)\n"
             "    elif figdir.startswith('mov'):\n"
             "        # Movies must be in _static\n"
             "        # Copy only the movies if they are needed through local filenames\n"
             "        if '\"'+ figdir in rst_text or '<' + figdir in rst_text:\n"
             "            destdir = os.path.join(source_dir, '_static', figdir)\n"
             "    if destdir is not None and os.path.isdir(figdir) and not os.path.isdir(destdir):\n"
             "        msg = 'copy: %%s to %%s' %% (figdir, destdir)\n"
             "        print(msg)\n"
             "        logging.info(msg)\n"
             "        shutil.copytree(figdir, destdir)\n\n"
             "# Copy needed figure files in current dir (not in fig* directories)\n"
             "for rstfile in glob.glob(os.path.join(source_dir, '*.rst')) + glob.glob(os.path.join(source_dir, '.*.rst')):\n"
             "    f = open(rstfile, 'r')\n"
             "    text = text_orig = f.read()\n"
             "    f.close()\n"
             "    import re\n"
             "    figfiles = [name.strip() for name in\n"
             "                re.findall('.. figure:: (.+)', text)]\n"
             "    local_figfiles = [name for name in figfiles if not os.sep in name]\n\n"
             "    for name in figfiles:\n"
             "        basename = os.path.basename(name)\n"
             "        if name.startswith('http') or name.startswith('ftp'):\n"
             "            pass\n"
             "        else:\n"
             "            if not os.path.isfile(os.path.join(source_dir, basename)):\n"
             "                msg = 'copy: %%s to %%s' %% (name, source_dir)\n"
             "                print(msg)\n"
             "                logging.info(msg)\n"
             "                shutil.copy(name, source_dir)\n"
             "            if name not in local_figfiles:\n"
             "                # name lies in another directory, make local reference to it\n"
             "                # since it is copied to source_dir\n"
             "                text = text.replace('.. figure:: %%s' %% name,\n"
             "                                    '.. figure:: %%s' %% basename)\n"
             "                logging.info('edit: figure path to %%s' %% basename)\n"
             "    if text != text_orig:\n"
             "        f = open(rstfile, 'w')\n"
             "        f.write(text)\n"
             "        f.close()\n\n"
             "# Copy linked local files, placed in _static*, to source_dir/_static\n"
             "staticdirs = glob.glob('_static*')\n"
             "for staticdir in staticdirs:\n"
             "    if os.listdir(staticdir):  # copy only if non-empty dir\n"
             "        cmd = 'cp -r %%(staticdir)s/* %%(source_dir)s/_static/' %% vars()\n"
             "        logging.info('running %%s' %% cmd)\n"
             "        system(cmd)\n\n"
             "# Create custom files in _static/_templates?\n"
             "if '**Show/Hide Code**' in rst_text:\n"
             "    with open(os.path.join(sphinx_rootdir, '_templates', 'page.html'), 'w') as f:\n"
             "        f.write(\"\"\"\n"
             "{%% extends \"!page.html\" %%}\n\n"
             "{%% set css_files = css_files + [\"_static/custom.css\"] %%}\n\n"
             "{%% block footer %%}\n"
             " <script type=\"text/javascript\">\n"
             "    $(document).ready(function() {\n"
             "        $(\".toggle > *\").hide();\n"
             "        $(\".toggle .header\").show();\n"
             "        $(\".toggle .header\").click(function() {\n"
             "            $(this).parent().children().not(\".header\").toggle(400);\n"
             "            $(this).parent().children(\".header\").toggleClass(\"open\");\n"
             "        })\n"
             "    });\n"
             "</script>\n"
             "{%% endblock %%}\n"
             "\"\"\")\n"
             "    with open(os.path.join(sphinx_rootdir, '_static', 'custom.css'), 'w') as f:\n"
             "        f.write(\"\"\"\n"
             ".toggle .header {\n"
             "    display: block;\n"
             "    clear: both;\n"
             "}\n\n"
             ".toggle .header:after {\n"
             "    content: \" ▼\";\n"
             "}\n\n"
             ".toggle .header.open:after {\n"
             "    content: \" ▲\";\n"
             "}\n"
             "\"\"\")\n\n") % vars())
    # (Note: must do cp -r above since shutil.copy/copytree cannot copy a la cp -r)
    f.write(("os.chdir(sphinx_rootdir)\n"
             "if '--runestone' not in sys.argv:\n"
             "    # Compile web version of the sphinx document\n"
             "    print(os.getcwd())\n"
             "    logging.info('running make clean and make html')\n"
             "    system('make clean')\n"
             "    system('make html')\n\n"
             "    print('Fix generated files:',)\n"
             "    os.chdir('_build/html')\n"
             "    for filename in glob.glob('*.html') + glob.glob('.*.html'):\n"
             "        print(filename)\n"
             "        f = open(filename, 'r'); text = f.read(); f.close()\n"
             "        text_orig = text\n"
             "        # Fix double title in <title> tags\n"
             "        text = re.sub(r'<title>(.+?) &mdash;.+?</title>', r'<title>\g<1></title>', text)\n"
             "        # Fix untranslated math (e.g. in figure captions and raw html)\n"
             "        text = re.sub(r':math:`(.+?)`', r' \( \g<1> \) ', text)\n"
             "        # Fix links to movies\n"
             "        text = re.sub(r'''src=['\"](mov.+?)['\"]''', r'src=\"_static/\g<1>\"', text)\n"
             "        # Fix movie frames in javascript player\n"
             "        text = text.replace(r'.src = \"mov', '.src = \"_static/mov')\n"
             "        # Fix admonition style\n"
             "        text = text.replace('</head>', '''\n"
             "       <style type=\"text/css\">\\n"
             "         div.admonition {\n"
             "           background-color: whiteSmoke;\n"
             "           border: 1px solid #bababa;\n"
             "         }\n"
             "       </style>\n"
             "      </head>\n"
             "    ''')\n\n"
             "        # Check if external links should pop up in separate windows\n"
             "        if '.. NOTE: Open external links in new windows.' in text:\n"
             "            text = text.replace('<a class=\"reference external\"',\n"
             "                                '<a class=\"reference external\" target=\"_blank\"')\n\n"
             "        # Make a link for doconce citation in copyright\n"
             "        if '. Made with DocOnce' in text:\n"
             "            text = text.replace('. Made with DocOnce', '')\n"
             "            text = text.replace('      Created using <a href=\"http://sphinx-doc.org/\">Sphinx', '      "
             "Created using <a href=\"https://github.com/hplgit/doconce\">DocOnce</a> and <a href=\"http://sphinx-doc.org/\">Sphinx')\n"
             "        # Remove (1), (2), ... numberings in identical headings\n"
             "        headings_wno = re.findall(r'(?<=(\d|\"))>([^>]+?)          \((\d+)\)<', text)\n"
             "        for dummy, heading, no in headings_wno:\n"
             "            heading = heading.strip()\n"
             "            text = re.sub(r'>%%s +\(%%d\)<' %%\n"
             "                          (re.escape(heading), int(no)),\n"
             "                          '>%%s<' %% heading, text)\n"
             "            text = re.sub(r'title=\"%%s +\(%%d\)\"' %%\n"
             "                          (re.escape(heading), int(no)),\n"
             "                          r'title=\"%%s\"' %% heading, text)\n\n"
             "        if os.path.isfile(filename + '.old~~'):\n"
             "            os.remove(filename + '.old~~')\n"
             "        f = open(filename, 'w'); f.write(text); f.close()\n"
             "        if text != text_orig:\n"
             "            logging.info('edit: %%s' %% filename)\n"
             "    os.chdir('../../')\n"             
             "    print('\\n\\ngoogle-chrome %(sphinx_rootdir)s/_build/html/index.html\\n')\n\n"
             "else:\n"
             "    # Add directory for RunestoneInteractive book\n"
             "    use_runestonebooks_style = True  # False: use user-chosen style\n"
             "    print(\"\"\"\n\ncreate RunestoneInteractive directory\n\"\"\")\n"
             "    sys.path.insert(0, os.curdir)\n"
             "    import conf as source_dir_conf  # read data from conf.py\n\n"
             "    if not os.path.isdir('RunestoneTools'):\n"
             "        system('git clone https://github.com/RunestoneInteractive/RunestoneComponents.git')\n"
             "    os.chdir('RunestoneComponents')\n"
             "    logging.info('creating RunestoneInteractive directory')\n\n"
             "    # Edit conf.py\n"
             "    # This one does not work anymore: run runestone init instead,\n"
             "    print('RunestoneInteractive has recently changed its setup - must abort')\n"
             "    sys.exit(1)\n"
             "    # it's the file runestone/__main__.py and function init()\n"
             "    # Need to build a bash script that runs the command and feeds the answers\n"
             "    # See also https://github.com/RunestoneInteractive/RunestoneComponents\n"
             "    f = open('conf.py.prototype', 'r');  text = f.read();  f.close()\n"
             "    text = text.replace('<ENTER YOUR PROJECT NAME HERE>', source_dir_conf.project)\n"
             "    text = text.replace('<INSERT YOUR PROJECT NAME HERE>', source_dir_conf.project)\n"
             "    text = text.replace('<ENTER YOUR COPYRIGHT NOTICE HERE>', source_dir_conf.copyright)\n"
             "    text = text.replace('<INSERT YOUR PROJECT NAME OR OTHER TITLE HERE>', source_dir_conf.project)\n"
             "    text = text.replace('<INSERT YOUR PROJECT NAME OR OTHER SHORT TITLE HERE>', source_dir_conf.project)\n"
             "    text = text.replace('html_theme_path = [\"_templates\"]', 'html_theme_path = [\"_templates\", \"../_themes\"]')\n"
             "    if not use_runestonebooks_style:\n"
             "        text = text.replace(\"html_theme = 'sphinx_bootstrap'\", \"html_theme = '%%s'\" %% source_dir_conf.html_theme)\n"
             "        text = re.sub(r'html_theme_options = \{.+?\}', 'html_theme_options = ' + str(source_dir_conf.html_theme_options) if "
             "hasattr(source_dir_conf, 'html_theme_options') else 'html_theme_options = {}', text, flags=re.DOTALL)\n"
             "    f = open('conf.py', 'w');  f.write(text);  f.close()\n\n"
             "    # Copy .rst files from sphinx dir\n"
             "    rst_files = [os.path.join(os.pardir, 'index.rst')] + glob.glob(os.path.join(os.pardir, '*.rst')) + "
             "glob.glob(os.path.join(os.pardir, '._*.rst'))\n"
             "    for filename in rst_files:\n"
             "        print('copying', filename, 'to _sources')\n"
             "        logging.info('copy: %%s to _sources' %% filename)\n"
             "        shutil.copy(filename, '_sources')\n"
             "    print('*** running paver build to build the RunestoneInteractive book')\n"
             "    logging.info('running paver build to create the RunestoneInteractive book')\n"
             "    system('paver build')\n\n"
             "    print('\\n\\ngoogle-chrome %(sphinx_rootdir)s/RunestoneTools/build/index.html')\n") % vars())
    f.close()
    os.chmod('automake_sphinx.py', 0o755)
    print(("'automake_sphinx.py' contains the steps to (re)compile the sphinx\n"
           "version. You may want to edit this file, or run the steps manually,\n"
           "or just run it by\n\n"
           "  python automake_sphinx.py\n"
           ))
    # Add scrollbars to navigation bar dropdown menu in case of bootstrap
    # (this css style is already written to index.rst)
    if 'bootstrap' in theme:
        with open(filename + '.rst', 'r') as f:
            text = f.read()
        # Trick to get scrollbar in dropdown menus
        text = (".. raw:: html\n\n"
                "  <style type=\"text/css\">\n"
                "    .dropdown-menu {\n"
                "      height: auto;\n"
                "      max-height: 400px;\n"
                "      overflow-x: hidden;\n"
                "  }\n"
                "  </style>\n\n") + text
        with open(filename + '.rst', 'w') as f:
            f.write(text)

def make_conf_py(themes, theme, title, short_title, copyright_,
                 logo, favicon, intersphinx):
    f = open('conf.py', 'r');  text = f.read();  f.close()

    # Compensate bug July 2016 in Sphinx
    text = text.replace('# import os', 'import os')

    # sphinx-quickstart sets copyright to authors with current year,
    # we customize this
    text = re.sub(r"copyright = u'(\d\d\d\d),.+",
                  r"copyright = u'%s'" % copyright_, text)

    # Propagate short_title
    pattern1 = '^short_title *='
    m = re.search(pattern1, text, flags=re.MULTILINE)
    if not m:
        # No short_title found, insert short_title after project
        pattern2 = r'(project *=.+)'
        if short_title == title:
            text = re.sub(pattern2, "\g<1>\nshort_title = project", text)
        else:
            text = re.sub(pattern2, "\g<1>\nshort_title = u'%s'" % short_title, text)
    else:
        if short_title != title:
            # User has given a short_title, must update short_title =
            text = re.sub(pattern1, "short_title = u'%s'" % short_title, text, flags=re.MULTILINE)

    pattern = r'#? *html_short_title *=.+'
    text = re.sub(pattern, 'html_short_title = short_title', text)
    # html_title defaults to "<project> v<release> documentation",
    # let it just be the title
    text = re.sub(r'# *html_title =.+', 'html_title = project', text)


    themes_list = ["html_theme = '%s'" % theme] + \
                  ["#html_theme = '%s'" % t for t in sorted(themes)]
    themes_code = (r"\ncheck_additional_themes= [\n"
                   r"   'solarized', 'cloud', 'redcloud',\n"
                   r"   'alabaster', 'bootstrap', 'impressjs']\n\n"
                   r"for theme in check_additional_themes:\n"
                   r"    if html_theme == theme:\n"
                   r"        if not theme in additional_themes_installed:\n"
                   r"            raise ImportError(\n"
                   r"                'html_theme = \"%s\", but this theme is not installed. %s' % (theme, additional_themes_url[theme]))\n\n\n\n"
                   r"if html_theme == 'solarized':\n"
                   r"    pygments_style = 'solarized'\n\n")
    text = re.sub(r"^html_theme = '.+?' *$",
                  '\n'.join(themes_list) + themes_code,
                  text, flags=re.MULTILINE)

    # Remove html_theme_path and html_static_path deep down in the conf.py
    # file because we need them earlier and must initialize them ourselves
    text = re.sub(r'# Add any paths that contain custom themes.+\n# *html_theme_path = \[\]', "", text)
    text = re.sub(r"# Add any paths that contain custom static.+?\n# *html_static_path = \['_static'\]", "", text)
    # Paths are added below when we try to import various themes in conf.py

    extensions = ("extensions = [\n"
                  "          #'sphinx.ext.pngmath',\n"
                  "          #'sphinx.ext.jsmath',\n"
                  "          'sphinx.ext.mathjax',\n"
                  "          #'matplotlib.sphinxext.mathmpl',\n"
                  "          #'matplotlib.sphinxext.only_directives',\n"
                  "          'matplotlib.sphinxext.plot_directive',\n"
                  "          'sphinx.ext.autodoc',\n"
                  "          'sphinx.ext.doctest',\n"
                  "          'sphinx.ext.viewcode',\n"
                  "          'sphinx.ext.intersphinx',\n"
                  "          'sphinx.ext.inheritance_diagram',\n"
                  "          'IPython.sphinxext.ipython_console_highlighting']\n\n"
                  "#pngmath_dvipng_args = ['-D 200', '-bg Transparent', '-gamma 1.5']  # large math fonts (200)\n\n"
                  "# Make sphinx aware of the DocOnce lexer\n"
                  "def setup(app):\n"
                  "    from sphinx.highlighting import lexers\n"
                  "    from doconce.misc import DocOnceLexer\n"
                  "    lexers['doconce'] = DocOnceLexer()\n\n"
                  "# Check which additional themes that are installed\n"
                  "additional_themes_installed = []\n"
                  "additional_themes_url = {}\n\n"
                  "# Add any paths that contain custom themes here, relative to this directory.\n"
                  "html_theme_path = ['_themes']\n"
                  "# Add any paths that contain custom static files (such as style sheets) here,\n"
                  "# relative to this directory. They are copied after the builtin static files,\n"
                  "# so a file named \"default.css\" will overwrite the builtin \"default.css\".\n"
                  "html_static_path = ['_static']\n\n"
                  "try:\n"
                  "    import alabaster\n"
                  "    additional_themes_installed.append('alabaster')\n"
                  "except ImportError:\n"
                  "    additional_themes_url['alabaster'] = 'pip install alabaster'\n\n"
                  "try:\n"
                  "    import sphinxjp.themes.solarized\n"
                  "    additional_themes_installed.append('solarized')\n"
                  "except ImportError:\n"
                  "    additional_themes_url['solarized'] = 'https://bitbucket.org/miiton/sphinxjp.themes.solarized: pip install --exists-action i -e hg+https://bitbucket.org/miiton/sphinxjp.themes.solarized#egg=sphinxjp.themes.solarized --upgrade'\n\n"
                  "try:\n"
                  "    import cloud_sptheme as csp\n"
                  "    additional_themes_installed.append('cloud')\n"
                  "    additional_themes_installed.append('redcloud')\n"
                  "except ImportError:\n"
                  "    url = 'https://bitbucket.org/ecollins/cloud_sptheme: pip install --exists-action i -e hg+https://bitbucket.org/ecollins/cloud_sptheme#egg=cloud_sptheme --upgrade'\n"
                  "    additional_themes_url['cloud'] = url\n"
                  "    additional_themes_url['redcloud'] = url\n\n\n"
                  "'''\n"
                  "# FIXME: think we do not need to test on basicstrap, but some themes\n"
                  "# need themecore and we must test for that\n"
                  "try:\n"
                  "    import sphinxjp.themecore\n"
                  "    if not 'sphinxjp.themecore' in extensions:\n"
                  "        extensions += ['sphinxjp.themecore']\n"
                  "    additional_themes_installed.append('basicstrap')\n"
                  "except ImportError:\n"
                  "    # Use basicstrap as an example on a theme with sphinxjp.themecore (??)\n"
                  "    additional_themes_url['basicstrap'] = 'https://github.com/tell-k/sphinxjp.themes.basicstrap: pip install --exists-action i -e git+https://github.com/ryan-roemer/sphinx-bootstrap-theme#egg=sphinx-bootstrap-theme --upgrade'\n"
                  "'''\n\n"
                  "try:\n"
                  "    import sphinxjp.themes.impressjs\n"
                  "    additional_themes_installed.append('impressjs')\n"
                  "except ImportError:\n"
                  "    additional_themes_url['impressjs'] = 'https://github.com/shkumagai/sphinxjp.themes.impressjs: pip install --exists-action i -e git+https://github.com/shkumagai/sphinxjp.themes.impressjs#egg=sphinxjp.themes.impressjs --upgrade'\n\n"
                  "try:\n"
                  "    import sphinx_bootstrap_theme\n"
                  "    additional_themes_installed.append('bootstrap')\n"
                  "except ImportError:\n"
                  "    additional_themes_url['bootstrap'] = 'https://github.com/ryan-roemer/sphinx-bootstrap-theme: pip install --exists-action i -e git+https://github.com/ryan-roemer/sphinx-bootstrap-theme#egg=sphinx-bootstrap-theme --upgrade'\n\n"
                  "try:\n"
                  "    import icsecontrib.sagecellserver\n"
                  "    extensions += ['icsecontrib.sagecellserver']\n"
                  "except ImportError:\n"
                  "    # pip install --exists-action i -e git+https://github.com/kriskda/sphinx-sagecell#egg=sphinx-sagecell --upgrade\n"
                  "    pass\n\n"
                  "# Is the document built on readthedocs.org? If so, don't import\n"
                  "on_rtd = os.environ.get('READTHEDOCS', None) == 'True'\n"
                  "if not on_rtd:  # only import and set the theme if we're building docs locally\n"
                  "    try:\n"
                  "        import sphinx_rtd_theme\n"
                  "        additional_themes_installed.append('sphinx_rtd_theme')\n"
                  "    except ImportError:\n"
                  "        additional_themes_url['sphinx_rtd_theme'] = 'pip install sphinx_rtd_theme'\n\n"
                  "tinker_themes = [\n"
                  "  'dark', 'flat', 'modern5', 'minimal5', 'responsive']\n"
                  "# http://tinkerer.me/index.html\n"
                  "# See Preview Another Theme in the sidebar of the above URL\n"
                  "try:\n"
                  "    import tinkerer\n"
                  "    import tinkerer.paths\n"
                  "    additional_themes_installed += tinker_themes\n"
                  "except ImportError:\n"
                  "    for theme in tinker_themes:\n"
                  "        additional_themes_url[theme] = 'pip install tinkerer --upgrade'\n\n")
    # Intersphinx mapping: the look up of URLs can take quite
    # some time, so this is not enabled by default
    if intersphinx:
        extensions += ("\n"
                       "#intersphinx_mapping = {}\n"
                       "# Example configuration for intersphinx for references to the\n"
                       "# the Sphinx documents for Python, NumPy, SciPy, Matplotlib.\n"
                       "# (Domos in http://scipy-lectures.github.com, typically :mod:`scipy.io`\n"
                       "# or :class:`numpy.ndarray` or :func:`math.asin`)\n"
                       "intersphinx_mapping = {\n"
                       "    'python': ('http://docs.python.org/2.7', None),\n"
                       "    'numpy': ('http://docs.scipy.org/doc/numpy', None),\n"
                       "    'scipy': ('http://docs.scipy.org/doc/scipy/reference', None),\n"
                       "    'mpl': ('http://matplotlib.org/', None),\n"
                       "}\n")

    #'matplotlib.sphinxext.ipython_directive',
    #'matplotlib.sphinxext.ipython_console_highlighting',
    # are now in IPython, but not installed as Python modules

    text = re.sub(r'extensions = .*?\]', extensions, text, flags=re.DOTALL)
    html_theme_options = (r"# See http://sphinx.pocoo.org/theming.html for options\n"
                          r"if html_theme in ('default', 'classic'):\n"
                          r"    # pygments_style =\n"
                          r"    html_theme_options = {\n"
                          r"       'rightsidebar': 'false',  # 'true'\n"
                          r"       'stickysidebar': 'false', # Make the sidebar \"fixed\" so that it doesn't scroll out of view for long body content.  This may not work well with all browsers.  Defaults to false.\n"
                          r"       'collapsiblesidebar': 'false', # Add an *experimental* JavaScript snippet that makes the sidebar collapsible via a button on its side. *Doesn't work together with \"rightsidebar\" or \"stickysidebar\".* Defaults to false.\n"
                          r"       'externalrefs': 'false', # Display external links differently from internal links.  Defaults to false.\n"
                          r"       # For colors and fonts, see default/theme.conf for default values\n"
                          r"       #'footerbgcolor':    # Background color for the footer line.\n"
                          r"       #'footertextcolor:'  # Text color for the footer line.\n"
                          r"       #'sidebarbgcolor':   # Background color for the sidebar.\n"
                          r"       #'sidebarbtncolor':  # Background color for the sidebar collapse button (used when *collapsiblesidebar* is true).\n"
                          r"       #'sidebartextcolor': # Text color for the sidebar.\n"
                          r"       #'sidebarlinkcolor': # Link color for the sidebar.\n"
                          r"       #'relbarbgcolor':    # Background color for the relation bar.\n"
                          r"       #'relbartextcolor':  # Text color for the relation bar.\n"
                          r"       #'relbarlinkcolor':  # Link color for the relation bar.\n"
                          r"       #'bgcolor':          # Body background color.\n"
                          r"       #'textcolor':        # Body text color.\n"
                          r"       #'linkcolor':        # Body link color.\n"
                          r"       #'visitedlinkcolor': # Body color for visited links.\n"
                          r"       #'headbgcolor':      # Background color for headings.\n"
                          r"       #'headtextcolor':    # Text color for headings.\n"
                          r"       #'headlinkcolor':    # Link color for headings.\n"
                          r"       #'codebgcolor':      # Background color for code blocks.\n"
                          r"       #'codetextcolor':    # Default text color for code blocks, if not set differently by the highlighting style.\n"
                          r"       #'bodyfont':         # Font for normal text.\n"
                          r"       #'headfont':         # Font for headings.\n"
                          r"    }\n\n"
                          r"elif html_theme == 'alabaster':\n"
                          r"    # Doc: https://pypi.python.org/pypi/alabaster\n"
                          r"    extensions += ['alabaster']\n"
                          r"    html_theme_path += [alabaster.get_path()]\n"
                          r"    html_theme_sidebars = {\n"
                          r"      '**': [\n"
                          r"        'about.html',\n"
                          r"        'navigation.html',\n"
                          r"        'relations.html',\n"
                          r"        'searchbox.html',\n"
                          r"        'donate.html',\n"
                          r"      ]\n"
                          r"    }\n\n"
                          r"elif html_theme == 'sphinx_rtd_theme':\n"
                          r"    # Doc: https://pypi.python.org/pypi/sphinx_rtd_theme\n"
                          r"    if not on_rtd:\n"
                          r"        html_theme_path += [sphinx_rtd_theme.get_html_theme_path()]\n\n"
                          r"elif html_theme == 'sphinxdoc':\n"
                          r"    # Doc: http://sphinx-doc.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'nosidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'solarized':\n"
                          r"    extensions += ['sphinxjp.themecore', 'sphinxjp.themes.solarized']\n\n"
                          r"elif html_theme in ('cloud', 'redcloud'):\n"
                          r"    html_theme_path += [csp.get_theme_dir()]\n\n"
                          r"elif html_theme == 'impressjs':\n"
                          r"    html_theme_path += [csp.get_theme_dir()]\n"
                          r"    if not 'sphinxjp.themecore' in extensions:\n"
                          r"        extensions += ['sphinxjp.themecore']\n\n"
                          r"elif html_theme == 'scrolls':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    pass\n"
                          r"    #html_theme_options = {\n"
                          r"       #'headerbordercolor':,\n"
                          r"       #'subheadlinecolor:',\n"
                          r"       #'linkcolor':,\n"
                          r"       #'visitedlinkcolor':\n"
                          r"       #'admonitioncolor':\n"
                          r"    #}\n\n"
                          r"elif html_theme == 'agogo':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    pass\n\n"
                          r"elif html_theme == 'nature':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'nosidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'traditional':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'nosidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'haiku':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'nosidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'pyramid':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'nosidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'bizstyle':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'nosidebar': 'false',  # 'true'\n"
                          r"       'rightsidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'epub':\n"
                          r"    # Doc: http://sphinx.pocoo.org/theming.html\n"
                          r"    html_theme_options = {\n"
                          r"       'relbar1': 'true',\n"
                          r"       'footer': 'true',\n"
                          r"    }\n\n"
                          r"elif html_theme == 'basicstrap':\n"
                          r"    html_theme_options = {\n"
                          r"       'rightsidebar': 'false',  # 'true'\n"
                          r"    }\n\n"
                          r"elif html_theme == 'bootstrap':\n"
                          r"    # Doc: https://ryan-roemer.github.io/sphinx-bootstrap-theme/README.html#customization\n"
                          r"    html_theme_options = {\n"
                          r"        'navbar_title': short_title,\n\n"
                          r"        # Global TOC depth for \"site\" navbar tab. (Default: 1)\n"
                          r"        # Switching to -1 shows all levels.\n"
                          r"        'globaltoc_depth': -1,\n\n"
                          r"        # HTML navbar class (Default: \"navbar\") to attach to <div> element.\n"
                          r"        # For black navbar, do \"navbar navbar-inverse\"\n"
                          r"        'navbar_class': \"navbar navbar-inverse\",\n\n"
                          r"        # Fix navigation bar to top of page?\n"
                          r"        # Values: \"true\" (default) or \"false\"\n"
                          r"        'navbar_fixed_top': \"true\",\n\n"
                          r"        # Location of link to source.\n"
                          r"        # Options are \"nav\" (default), \"footer\" or anything else to exclude.\n"
                          r"        'source_link_position': \"footer\",\n\n"
                          r"        # Any Bootswatch theme (http://bootswatch.com/) can be used\n"
                          r"        #'bootswatch_theme': 'readable',\n\n"
                          r"        # A list of tuples containing pages or urls to link to.\n"
                          r"        # Valid tuples should be in the following forms:\n"
                          r"        #    (name, page)                 # a link to a page\n"
                          r"        #    (name, \"/aa/bb\", 1)          # a link to an arbitrary relative url\n"
                          r"        #    (name, \"http://example.com\", True) # arbitrary absolute url\n"
                          r"        # Note the \"1\" or \"True\" value above as the third argument to indicate\n"
                          r"        # an arbitrary url.\n"
                          r"        #'navbar_links': [('PDF', '../mydoc.pdf', True), ('HTML', '../mydoc.html', True)],\n\n"
                          r"        # TODO: Future.\n"
                          r"        # Add page navigation to it's own navigation bar.\n"
                          r"        #'navbar_page_separate': True,\n"
                          r"    }\n"
                          r"    html_theme_path += sphinx_bootstrap_theme.get_html_theme_path()\n\n"
                          r"elif html_theme == 'scipy_lectures':\n"
                          r"    # inherits the default theme and has all those options\n"
                          r"    # set rightsidebar to true and nodesidebar to true to get\n"
                          r"    # sidebar with the matching colors\n"
                          r"    html_theme_options = {\n"
                          r"        'nosidebar': 'true',\n"
                          r"        'rightsidebar': 'false',\n"
                          r"        'sidebarbgcolor': '#f2f2f2',\n"
                          r"        'sidebartextcolor': '#20435c',\n"
                          r"        'sidebarlinkcolor': '#20435c',\n"
                          r"        'footerbgcolor': '#000000',\n"
                          r"        'relbarbgcolor': '#000000',\n"
                          r"    }\n\n"
                          r"elif html_theme == 'cbc':\n"
                          r"    pygments_style = \"friendly\"\n"
                          r"elif html_theme == 'uio':\n"
                          r"    pygments_style = \"tango\"\n"
                          r"elif html_theme in tinker_themes:\n"
                          r"    html_theme_options = {}\n"
                          r"    extensions += ['tinkerer.ext.blog', 'tinkerer.ext.disqus']\n"
                          r"    html_static_path += [tinkerer.paths.static]\n"
                          r"    html_theme_path += [tinkerer.paths.themes]\n\n\n\n")
    text = re.sub(r'^# *html_theme_options = \{\}', html_theme_options,
                  text, flags=re.MULTILINE)
    #text = re.sub('^# *html_theme_path = \[\]', html_theme_options,

    # Examples on settings: http://pylit.berlios.de/conf.py.html

    text = re.sub(r'# *html_use_index =.+', ("if html_theme == 'impressjs':\n"
                                             "    html_use_index = False\n"), text)

    if logo is not None:
        # Must copy logo file
        logo_basename = os.path.basename(logo)
        shutil.copy(os.path.abspath(logo), logo_basename)
        text = re.sub(r'# *html_logo = None', 'html_logo = "%s"'
                      % logo_basename, text)

    if favicon is not None:
        # Must copy logo file
        favicon_basename = os.path.basename(favicon)
        shutil.copy(os.path.abspath(favicon),
                    os.path.join('_static', favicon_basename))
        text = re.sub(r'# *html_favicon = None', 'html_favicon = "%s"'
                      % favicon_basename, text)

    # For tinker themes we need to copy index.rst to master.rst
    if theme in ['dark', 'flat', 'modern5', 'minimal5', 'responsive']:
        shutil.copy('index.rst', 'master.rst')
        text = text.replace("master_doc = 'index'", "master_doc = 'master'")
        text = re.sub(r'(html_title =.+)', (r'\g<1>\n'
                                            r'tagline = ""  # subtitle\n'
                                            r'html_add_permalinks = None', text))

    f = open('conf.py', 'w');  f.write(text);  f.close()

    # Write _templates/layout.html file for sagecellserver boxes
    try:
        import icsecontrib.sagecellserver
        f = open(os.path.join('_templates', 'layout.html'), 'w')
        f.write(("{% extends \"!layout.html\" %}\n"
                 "{% block linktags %}\n\n"
                 "        <script src=\"http://sagecell.sagemath.org/static/jquery.min.js\"></script>\n"
                 "        <script src=\"http://sagecell.sagemath.org/static/embedded_sagecell.js\"></script>\n\n"
                 "        <script>sagecell.makeSagecell({inputLocation: \".sage\"});</script>\n\n"
                 "        <style type=\"text/css\">\n"
                 "                .sagecell .CodeMirror-scroll {\n"
                 "                        overflow-y: hidden;\n"
                 "                        overflow-x: auto;\n"
                 "                }\n"
                 "                .sagecell .CodeMirror {\n"
                 "                        height: auto;\n"
                 "                }\n"
                 "        </style>\n\n"
                 "    {{ super() }}\n"
                 "{% endblock %}\n"))
        f.close()
    except ImportError:
        pass


def make_conf_py_runestone(themes, theme, title, short_title,
                           copyright_, logo, favicon, intersphinx):
    f = open('conf.py', 'r');  text = f.read();  f.close()
    text = re.sub(r"^project = u'<ENTER.+", "project = u'%s'" % title,
                  flags=re.MULTILINE)
    text = re.sub(r"^copyright = u'<ENTER.+", "copyright = u'%s'" % copyright_,
                  flags=re.MULTILINE)
    text = re.sub(r"'navbar_title':.+", "'navbar_title': '%s'" % title)
    f = open('conf.py', 'w');  f.write(text);  f.close()




# -----------------------------------------------------------------------


# Variables to be shared across modules

dofile_basename = None
filename = None

encoding = ''

_log = ''
_log_filename = ''

supported_format_names = ['html', 'latex', 'pdflatex', 'rst', 'sphinx', 'st', 'epytext',
                          'plain', 'gwiki', 'mwiki', 'cwiki', 'pandoc', 'ipynb', 'matlabnb']

# Begin-end environments
doconce_envirs = ['c', 't',                # verbatim and tex blocks
            'ans', 'sol', 'subex',        # exercises
            'ans_docend', 'sol_docend',   # exercises at document end
            'pop', 'slidecell', 'notes',  # slides
            'hint', 'remarks',            # exercises
            'quote', 'box',
            'notice', 'summary', 'warning', 'question', 'block', # admon
            'quiz', 'u-',
            ]

main_content_char = '-'

# Python colors and styling for printing
style = {
    'green': '\033[92m',
    'red': '\033[91m',
    'bold': '\033[1m',
    'underline': '\033[4m',
    'ybackground': '\033[33m',
    'black': '\033[30m',
    'yellow': '\033[33m',
    'magenta': '\033[35',
    'white': '\033[37m',
    'blue': '\033[94m',
    '_end': '\033[0m'
}

# Regular expressions for inline tags:
inline_tag_begin = r"""(?P<begin>(^|[(\s~>{!-]|^__|&[mn]dash;))"""
inline_tag_end = r"""(?P<end>($|[.,?!;:)<}!'\s~\[<&;-]))"""

# Admonitions
admons = 'notice', 'summary', 'warning', 'question', 'block'

# Support for non-English languages (not really implemented yet)
locale_dict = dict(
    language='English',  # language to be used
    American={
        # 'English' is an alias for 'American'
        'locale': 'us_US.UTF-8',
        'latex package': 'english',
        'aspell_dictionary' : 'american', # with aspell, this is an alias for 'en_US'
        'toc': 'Table of contents',
        'Contents': 'Contents',
        'Figure': 'Figure',
        'Movie': 'Movie',
        'list of': 'List of',
        'and': 'and',
        'Exercise': 'Exercise',
        'Project': 'Project',
        'Problem': 'Problem',
        'Example': 'Example',
        'Projects': 'Projects',
        'Problems': 'Problems',
        'Examples': 'Examples',
        'Preface': 'Preface',
        'Abstract': 'Abstract',
        'Summary': 'Summary',
        # Admons
        'summary': 'summary',
        'hint': 'hint',
        'question': 'question',
        'notice': 'notice',
        'warning': 'warning',
        # box, quote are silent wrt title
        'remarks': 'remarks', # In exercises
        # Exercise headings
        'Solution': 'Solution',
        'Answer': 'Answer',
        'Hint': 'Hint',
        # At the end (in Sphinx)
        'index': 'Index',
        # References
        'Filename': 'Filename',
        'Filenames': 'Filenames',
        # Quiz
        'question_prefix': 'Question:', # question in multiple-choice quiz
        'choice_prefix': 'Choice',      # choice in multiple-choice quiz
    },
    Norwegian={
        'locale': 'nb_NO.UTF-8', # norsk bokmål
        'latex package': 'norsk',
        'aspell_dictionary' : 'norsk', # with aspell, this is an alias for 'nb'
        'toc': 'Innholdsfortegnelse',
        'Contents': 'Innhold',
        'Figure': 'Figur',
        'Movie': 'Video',
        'list of': 'Liste over',
        'and': 'og',
        'Exercise': 'Oppgave',
        'Project': 'Prosjekt',
        'Problem': 'Problem',
        'Example': 'Eksempel',
        'Exercises': 'oppgaver',
        'Projects': 'prosjekter',
        'Problems': 'problemer',
        'Examples': 'eksempler',
        'Preface': 'Forord',
        'Abstract': 'Sammendrag',
        'Summary': 'Sammendrag',
        'summary': 'sammendrag',
        'hint': 'Hint',
        'question': u'spørsmål',
        'notice': u'observér',
        'warning': 'advarsel',
        'remarks': 'bemerkning',
        'index': 'Stikkordsliste',
        'Solution': u'Løsningsforslag',  # In exercises
        'Answer': 'Fasit',  # In exercises
        'Hint': 'Hint',  # In exercises
        'Filename': 'Filnavn',
        'Filenames': 'Filnavn',
        # Quiz
        'question_prefix': u'Spørsmål:',    # question in multiple-choice quiz
        'choice_prefix': 'Alternativ',      # choice in multiple-choice quiz
    },
    German={
        'locale': 'de_DE.UTF-8',
        'latex package': 'german',
        'aspell_dictionary' : 'deutsch', # with aspell, this is an alias for 'de_DE'
        'toc': 'Inhaltsverzeichnis',
        'Contents': 'Inhalt',
        'Figure': 'Abbildung',
        'Movie': 'Film',
        'list of': 'Liste von',
        'and': 'und',
        'Exercise': u'Übung',
        'Project': 'Projekt',
        'Problem': 'Problem',
        'Example': 'Beispiel',
        'Projects': 'Projekte',
        'Problems': 'Probleme',
        'Examples': 'Beispiele',
        'Preface': 'Vorwort',
        'Abstract': 'Abstract',
        'Summary': 'Zusammenfassung',
        # Admons
        'summary': 'Zusammenfassung',
        'hint': 'Hinweis',
        'question': 'Frage',
        'notice': 'Notiz',
        'warning': 'Warnung',
        # box, quote are silent wrt title
        'remarks': 'Bemerkung', # In exercises
        # Exercise headings
        'Solution': u'Lösung',
        'Answer': 'Antwort',
        'Hint': 'Hinweis',
        # At the end (in Sphinx)
        'index': 'Index',
        # References
        'Filename': 'Dateiname',
        'Filenames': 'Dateiname',
    },
    Basque={
	    'locale': 'eu_ES.UTF-8',
	    'latex package': 'basque',
	    'toc': 'Aurkibidea',
	    'Contents': 'Edukiak',
	    'Figure': 'Irudia',
	    'Movie': 'Filma',
	    'list of': 'Zerrenda',
	    'and': 'eta',
	    'Exercise': 'Ariketa',
	    'Project': 'Proiektua',
	    'Problem': 'Problema',
	    'Example': 'Adibidea',
	    'Projects': 'Proiektuak',
	    'Problems': 'Problemak',
	    'Examples': 'Adibideak',
	    'Preface': 'Hitzaurrea',
	    'Abstract': 'Laburpena',
	    'Summary': 'Laburpena',
	    # Admons
	    'summary': 'laburpena',
	    'hint': 'laguntza',
	    'question': 'galdera',
	    'notice': 'oharra',
	    'warning': 'kontuz',
	    # box, quote are silent wrt title
	    'remarks': 'iruzkinak', # In exercises
	    # Exercise headings
	    'Solution': 'Soluzioa',
	    'Answer': 'Erantzuna',
	    'Hint': 'Laguntza',
	    # At the end (in Sphinx)
	    'index': 'Indizea',
	    # References
	    'Filename': 'Fitxategi',
	    'Filenames': 'Fitxategiak',
    },
    Arabic={
        'locale': 'ar_SA.UTF-8',
        'latex package': 'arabic',
        'aspell_dictionary' : 'arabic', # with aspell, this is an alias for 'ar'
        'toc': u'الفَهْرس',
        'Contents': u'المُحْتويات',
        'Figure': u'رَسْم تَوضِيحي',
        'Movie': u'فِيلم',
        'list of': u'قَائِمة',
        'and': u'و',
        'Exercise': u'تَمْرِين',
        'Project': u'مَشْرُوع',
        'Problem': u'مَسْألة',
        'Example': u'مِثَال',
        'Projects': u'مَشَارِيع',
        'Problems': u'مَسَائِل',
        'Examples': u'أَمْثِلة',
        'Preface': u'مُقَدِّمة',
        'Abstract': u'المُلَّخص',
        'Summary': u'الخُلاصَة',
        # Admons
        'summary': u'الخُلاصة',
        'hint': u'تَلْمِيح',
        'question': u'سُؤال',
        'notice': u'تَنْبِيه',
        'warning': u'تَحْذِير',
        # box, quote are silent wrt title
        'remarks': u'مُلاحَظات', # In exercises
        # Exercise headings
        'Solution': u'الحَل',
        'Answer': u'الجَواب',
        'Hint': u'تَلْميح',
        # At the end (in Sphinx)
        'index': u'الدَّليل',
        # References
        'Filename': u'اسم_الملف',
        'Filenames': u'أسماء_الملفات',
    }
    )
# Let English be an alias for American
locale_dict['English'] = locale_dict['American'].copy()
# Create British based on (American) English
locale_dict['British'] = locale_dict['American'].copy()
locale_dict['British']['locale'] = 'en_GB.UTF-8'
locale_dict['British']['aspell_dictionary'] = 'british' # with aspell, this is an alias for 'en_GB'
locale_dict['British']['latex package'] = 'british' # with aspell, this is an alias for 'en_GB'

def lookup_locale_dict(key, fallback_to_english=True):
    if key in locale_dict[locale_dict['language']]:
        return locale_dict[locale_dict['language']][key]

    # could not find localized term
    if fallback_to_english:
        assert key in locale_dict['English'], "'%s' is not in locale_dict" % key
        return locale_dict['English'][key]
    else:
        raise ValueError("'%s' is not in locale_dict for %s" % (key, locale_dict['language']))

_registered_command_line_options = [
    ('--help', 'Print all command-line options for doconce'),
    ('--<cmd-option> --help', 'Print a specific command-line option <cmd-option> for doconce'),
    ('--debug', """Write a debugging file _doconce_debugging.log with lots of intermediate results"""),
    ('--no_abort', 'Do not abort the execution if syntax errors are found.'),
    ('--verbose=', """Write progress of intermediate steps if they take longer than X seconds. 0: X=15 (default); 1: X=5; 2: X=0"""),
    ('--language=', """Native language to be used: English (default), Norwegian, German, Basque, Arabic"""),
    ('--preprocess_include_subst', """Turns on variable substitutions in # #include paths when running Preprocess: preprocess -i -DMYDIR=rn1 will lead to the string "MYDIR" being replaced by the value "rn1" in # #include "..." statements."""),
    ('--syntax_check=', """Values: on/off. Turns on/off fix of illegal constructions and the syntax check (may be time consuming for large books)."""),
    ('--skip_inline_comments', 'Remove all inline comments of the form [ID: comment].'),
    ('--draft', 'Indicates draft (turns on draft elements in LaTeX, otherwise no effect).'),
    ('--CC_license=', """Template wording for Creative Commons licenses. Default: "Released under CC %s 4.0 license." Example: "This work is released under the Creative Commons %s 4.0 license". CC license is specified as a part of the copyright syntax, e.g.: "AUTHOR: Kaare Dump {copyright|CC BY} at BSU & Some Company Ltd"; or:  "AUTHOR: Kaare Dump at BSU & Some Company Ltd. {copyright,2005-present|CC BY-NC}". The --CC_license= option has no effect if the license does not start with CC, e.g.: "AUTHOR: Kaare Dump at BSU {copyright|Released under the MIT license.}\""""),
    ('--align2equations', """Rewrite align/alignat math environments to separate equation environments. Sometimes needed for proper MathJax rendering (e.g., remark slides). Sphinx requires such rewrite and will do it regardless of this option."""),
    ('--force_tikz_conversion', 'Force generation SVG/HTML versions of tikz figures, overwriting any previously generated SVG/HTML files (applies to all formats except LaTeX)'),
    ('--tikz_libs=', 'TikZ libraries used in figures.'),
    ('--pgfplots_libs=', 'pgfplots libraries used in figures.'),
    ('--IBPLOT', 'automagic translation of IBPLOT commands.'),
    ('--exercise_numbering=', """absolute: exercises numbered as 1, 2, ... (default); chapter: exercises numbered as 1.1, 1.2, ... , 3.1, 3.2, ..., B.1, B.2, etc. with a chapter or appendix prefix."""),
    ('--exercises_in_zip', 'Place each exercises as an individual DocOnce file in a zip archive.'),
    ('--exercises_in_zip_filename=', """Filenames of individual exercises in zip archive. logical: use the (first) logical filename specified by file=... ; number:  use either absolute exercise number or chapter.localnumber."""),
    ('--toc_depth=', """No of levels in the table of contents. Default: 2, which means chapters, sections, and subsections. Set to 1 to exclude subsections. Applies to all formats, except sphinx: for sphinx, set toc_depth=... as part of the command doconce sphinx_dir."""),
    ('--encoding=', 'Specify encoding (e.g., latin1 or utf-8).'),
    ('--no_ampersand_quote', 'Turn off special treatment of ampersand (&). Needed, e.g., when native latex code for tables are inserted in the document.'),
    ('--no_mako', 'Do not run the Mako preprocessor program.'),
    ('--no_preprocess', 'Do not run the Preprocess preprocessor program.'),
    ('--mako_strict_undefined', 'Make Mako report on undefined variables.'),
    ('--no_header_footer', 'Do not include header and footer in (LaTeX and HTML) documents.'),
    ('--no_emoji', 'Remove all emojis.'),
    ('--siunits', 'Allow siunitx MathJax/LaTeX package for support of SI units in various formats'),
    ('--allow_refs_to_external_docs', 'Do not abort translation if ref{...} to labels not defined in this document.'),
    ('--userdef_environment_file=', 'Read user-defined environments from this file instead of the default (userdef_environments.py)'),
    ('--runestone', 'Make a RunestoneInteractive version of a Sphinx document.'),
    ('--max_bc_linelength=', """Strip lines in !bc environments that are longer than specified (to prevent too long lines). Default: None (no length restriction)."""),
    ('--replace_ref_by_latex_auxno=', """Replace all ref{...} by hardcoded numbers from a latex .aux file. Makes it possible for a notebook or html page to refer to a latex textbook. Recommended syntax: see (ref{my:eq1}) in cite{MyBook}, or see Section ref{my:sec2} in cite{MyBook}."""),
    ('--keep_pygments_html_bg', """Do not allow change of background in code blocks in HTML."""),
    ('--minted_latex_style=', """Specify the minted style to be used for typesetting code in LaTeX. See pygmetize -L styles for legal names."""),
    ('--pygments_html_style=', """Specify the minted/pygments style to be used for typesetting code in HTML. Default: default (other values: monokai, manni, rrt, perldoc, borland, colorful, murphy, trac, tango, fruity, autumn, emacs, vim, pastie, friendly, native, see pygmentize -L styles). none, no, off: turn off pygments to typeset computer code in HTML, use plain <pre> tags. highlight.js: use highlight.js syntax highlighting, not pygments."""),
    ('--pygments_html_linenos', """Turn on line numbers in pygmentized computer code in HTML. (In LaTeX line numbers can be added via doconce subst or doconce replace such that the verbatim environments get the linenos=true parameter.)"""),
    ('--xhtml', 'Use BeautifulSoap to try to produce XHTML output. It inserts end tags (e.g. </p>) and guesses where to do it.'),
    ('--html_output=', 'Alternative basename of files associated with the HTML format.'),
    ('--html_style=', """Name of theme for HTML style: plain, blueish, blueish2, bloodish, tactile-black, tactile-red, rossant solarized, solarized2_light, solarized2_dark, bootstrap, bootswatch, bootstrap_X (with  X=bloodish, blue, bluegray, brown, cbc, FlatUI, red), bootswatch_X (with X=cerulean, cosmo, flatly, journal, lumen, readable, simplex, spacelab, united, yeti (dark:) amelia, cyborg, darkly, slate, spruce, superhero (demos at bootswatch.com))"""),
    ('--html_template=', """Specify an HTML template with header/footer in which the doconce document is embedded. (Often preferred to run with --no_title)"""),
    ('--no_title', 'Comment out TITLE, AUTHOR, DATE. Often used with HTML templates.'),
    ('--html_code_style=', """off, inherit, or transparent: enable normal inline verbatim font where foreground and background color is inherited from the surroundnings. off, inherit and transparent are just synonyms for inheriting color from the text and make the background color transparent (use e.g. --html_code_style=inherit to avoid the red Boostrap color). Default: on (use the css-specified typesetting of <pre> tags). NOTE: the naming "html_code_style" is not optimal: it has nothing to do with code block style, but the <code> tag for inline verbatim text in the context of bootstrap css styles."""),
    ('--html_pre_style=', """off, inherit, or transparent: let code blocks inside <pre> tags have foreground and background color inherited from the surroundnings. Default: on (use the css-specified typesetting of <pre> tags). This option is most relevant for Bootstrap styles to avoid white background in code blocks inside colorful admons."""),
    ('--html_toc_indent=', """No of spaces for indentation of subsections in the table of contents in HTML output. Default: 3 (0 gives toc as nested list in Bootstrap-based styles)."""),
    ('--html_body_style=', """Override elements in the <body> style css. Used to enlargen bootswatch fonts, for instance: "--html_body_style=font-size:20px;line-height:1.5\""""),
    ('--html_body_font=', """Specify HTML font for text body. =? lists available fonts."""),
    ('--html_heading_font=', """Specify HTML font for headings. =? lists available fonts."""),
    ('--html_video_autoplay=', """True for autoplay when HTML is loaded, otherwise False (default)."""),
    ('--html_admon=', """Type of admonition and color: colors, gray, yellow, apricot, lyx, paragraph. For html_style=bootstrap*,bootswatch*, the two legal values are boostrap_panel, bootstrap_alert."""),
    ('--html_admon_shadow', 'Add a shadow effect to HTML admon boxes (gray, yellow, apricot).'),
    ('--html_admon_bg_color=', 'Background color of admon in HTML.'),
    ('--html_admon_bd_color=', 'Boundary color of admon in HTML.'),
    ('--css=', """Specify a .css style file for HTML output. If the file does not exist, the default or specified style (--html_style=) is written to it."""),
    ('--html_box_shadow', 'Add a shadow effect in HTML box environments.'),
    ('--html_share=', """Specify URL and there will be Facebook, Twitter, etc. buttons at the end of the HTML document. --html_share=http://mysite.com/specials shares on email, Facebook, Google+, LinkedIn, Twitter, and enables a print button too. --html_share=http://mysite.com/specials,twitter,facebook shares on Twitter and Facebook only. Sites are separated by comma. The following names are allowed: email, facebook, google+, linkedin, twitter, print."""),
    ('--html_exercise_icon=', """Specify a question icon (as a filename in the bundled/html_images directory in the doconce repo) for being inserted to the right in exercises. default: turn on predefined question icons according to the chosen style. none: no icons (this is the default value)."""),
    ('--html_exercise_icon_width=', """Width of the icon image in pixels (must be used with --html_exercise_icon)."""),
    ('--html_raw_github_url=', """URLs to files hosted on the doconce github account. Internet Explorer (and perhaps other browsers) will not show raw.github.com files. Instead on should use rawgit.com. For development of HTML sites in Safari and Chrome and can use rawgit.com. \n  Values of --html_raw_github_url=: safe or cdn.rawgit: use this for ready-made sites with potentially some traffic. The URL becomes https://cdn.rawgit.com/doconce/doconce/...\n  test or rawgit: use this for test purposes and development with low traffic. The URL becomes https://rawgit.com/doconce/doconce/... \n  github or raw.github: the URL becomes https://raw.github.com and may fail to load properly. \n  githubusercontent or raw.githubusercontent: The URL becomes https://raw.githubusercontent.com and may fail to load properly."""),
    ('--html_DOCTYPE', """Insert <!DOCTYPE HTML> in the top of the HTML file. This is required for Internet Explorer and Mozilla. However, some of the CSS files used by DocOnce may not load properly if they are not well formed. That is why no doctype is default in the generated HTML files."""),
    ('--html_links_in_new_window', """Open HTML links in a new window/tab."""),
    ('--html_quiz_button_text=', """Text on buttons for collapsing/expanding answers and explanations in quizzes (with bootstrap styles). Default: Empty (just pencil glyphion)."""),
    ('--html_bootstrap_navbar=', 'Turns the Bootstrap navigation bar on/off. Default: on.'),
    ('--html_bootstrap_jumbotron=', """Turns the Bootstrap jumbotron intro on/off and governs the size of the document title. Default: on. Other values: h2, off (h2 gives h2 heading instead of h1, off gives no jumbotron)."""),
    ('--html_bootstrap_navbar_links=', """Allows custom links in the navigation bar. Format: link|url;link|url;link|url . Example: "--html_bootstrap_navbar_links=Google|http://google.com;hpl|http://folk.uio.no/hpl\""""),
    ('--html_figure_caption=', """Placement of figure caption: top (default) or bottom. (sidecap=True is another option, this can be set for individual figures, while --html_figure_caption controls the general caption placement of all figures."""),
    ('--html_figure_hrule=', """Set horizontal rule(s) above and/or below a figure. top: rule at top (default); none, off: no rules; bottom: rule at bottom; top+bottom: rule at top and bottom"""),
    ('--html_copyright=', 'Controls where to put copyright statements. everypage: in the footer of every page; titlepages or titlepage: in the footer of the titlepage only (default).'),
    ('--cite_doconce', 'Adds a citation to the DocOnce web page if copyright statements are present.'),
    ('--device=', """Set device to paper, screen, or other (paper impacts LaTeX output)."""),
    ('--number_all_equations', """Switch latex environments such that all equations get a number."""),
    ('--denumber_all_equations', """Switch latex environments such no equations get a number (useful for removing equation labels in slides). Error messages are issued about references to numbered equations in the text."""),
    ('--latex_style=',
     """LaTeX style package used for the document.
  std: standard LaTeX article or book style,
  Springer_sv: Springer's svmono class (the new standard for all Springer books),
  Springer_T2: Springer's T2 book style,
  Springer_T4: Springer's T4 book style (smaller pagesize than T2),
  Springer_lncse: Springer's Lecture Notes in Computational Science and Engineering (LNCSE) style,
  Springer_llncs: Springer's Lecture Notes in Computer Science style,
  Springer_lnup:  Springer's Lecture Notes in University Physics,
  Springer_collection: Springer's style for chapters in LNCSE proceedings,
  tufte-book: use of tufte-book.cls for E. Tufte-inspired layout,
  Koma_Script: Koma Script style,
  siamltex: SIAM's standard LaTeX style for papers,
  siamltexmm: SIAM's extended (blue) multimedia style for papers.
  elsevier: Elsevier Style"""),
    ('--latex_font=', """LaTeX font choice: helvetica, palatino, utopia, std (Computer Modern, default)."""),
    ('--latex_code_style=', """
  Typesetting of code blocks.
    pyg: use pygments (minted), style is set with --minted_latex_style=
    lst: use lstlistings
    vrb: use Verbatim (default)
  Specifications across languages:
    pyg-blue1
    lst, lst-yellowgray[style=redblue]
    vrb[frame=lines,framesep=2.5mm,framerule=0.7pt]
  Detailed specification for each language:
    default:vrb-red1[frame=lines]@pycod:lst[style=redblue]@pypro:lst-blue1[style=default]@sys:vrb[frame=lines,label=\\fbox{{\\tiny Terminal}},framesep=2.5mm,framerule=0.7pt]
  Here, Verbatim[frame=lines] is used for all code environments, except pycod, pypro and sys, which have their own specifications.
    pycod: lst package with redblue style (and white background)
    pypro: lst package with default style and blue1 background
    style, sys: Verbatim with the specified arguments and white background.
  (Note: @ is delimiter for the language specifications, syntax is envir:package-background[style parameters]@)"""),
    ('--latex_code_leftmargin=', 'Sets the left margin in code blocks. Default: 7 (mm).'),
    ('--latex_code_bg=', 'Background color code blocks. Default: white.'),
    ('--latex_code_bg_vpad', 'Vertical padding of background. Has only effect for vrb/pyg-bgcolor styles (not lst!).'),
    ('--latex_code_lststyles=', """Filename with LaTeX definitions of lst styles."""),
    ('--latex_copyright=', 'Controls where to put copyright statements. everypage: in the footer of every page; titlepages: in the footer of the titlepage and chapter pages (for books) only (default).'),
    ('--latex_bibstyle=', 'LaTeX bibliography style. Default: plain.'),
    ('--section_numbering=', 'Turn section numbering on/off. Default: off for all formats except latex and pdflatex (on for those).'),
    ('--latex_table_format=', 'Default: quote. Other values: left, center, footnotesize, tiny.'),
    ('--latex_table_row_sep=', r'Row separation factor in tables (command \renewcommand{\arraystretch}{<factor>}. Default: 1.0'),
    ('--latex_title_layout=', """Layout of the title, authors, and date: std: traditional LaTeX layout; titlepage: separate page; doconce_heading (default): authors with "footnotes" for institutions; beamer: layout for beamer slides."""),
    ('--latex_link_color=', """Color used in hyperlinks. Default is dark blue if --device=screen, or black if --device=paper (invisible in printout or special blue color if --latex_section_headings=blue or strongblue). Values are specified either as comma-separated rgb tuples or as color names, e.g., --latex_link_color=0.1,0.9,0.85 or --latex_link_color=red or --latex_link_color=gray!70"""),
    ('--latex_title_reference=', """latex code placed in a footnote for the title, typically used for acknowledging publisher/source of original version of the document."""),
    ('--latex_encoding=', 'Encoding for \\usepackage[encoding]{inputenc}. Values: utf8 (default) or latin1.'),
    ('--latex_packages=', """Comma-separated list of latex packages to be included in \\usepackage commands.."""),
    ('--latex_papersize=', """Geometry of page size: a6, a4, std (default)."""),
    ('--latex_list_of_exercises=', """LaTeX typesetting of list of exercises: loe: special, separate list of exercises; toc: exercises included as part of the table of contents; none (default): no list of exercises."""),
    ('--latex_movie=', """Specify package for handling movie/video content. Default: href (hyperlink to movie file). Other options: media9, movie15, multimedia (Beamer's \\movie command)."""),
    ('--latex_movie_controls=', 'Specify control panel for movies. Default: on. Other options: off.'),
    ('--latex_external_movie_viewer', 'Allow external movie viewer for movie15 package.'),
    ('--latex_fancy_header', """Typesetting of headers on each page: If article: section name to the left and page number to the right on even page numbers, the other way around on odd page numners. If book: section name to the left and page numner to the right on even page numbers, chapter name to the right and page number to the left on odd page numbers."""),
    ('--latex_section_headings=', """Typesetting of title/section/subsection headings: std (default): standard LaTeX; blue: gray blue color; strongblue: stronger blue color; gray: white text on gray background, fit to heading width; gray-wide: white text on gray background, wide as the textwidth."""),
    ('--latex_colored_table_rows=', """Colors on every two line in tables: no (default), gray, blue."""),
    ('--latex_line_numbers', """Include line numbers for the running text (only active if there are inline comments."""),
    ('--latex_todonotes', """Use the todonotes package to typeset inline comments. Gives colored bubbles in the margin for small inline comments and in the text for larger comments."""),
    ('--latex_double_spacing', """Sets the LaTeX linespacing to 1.5 (only active if there are inline comments)."""),
    ('--latex_labels_in_margin', """Print equation, section and other LaTeX labels in the margin."""),
    ('--latex_index_in_margin', 'Place entries in the index also in the margin.'),
    ('--latex_preamble=', """User-provided LaTeX preamble file, either complete or additions to the doconce-generated preamble."""),
    ('--latex_no_program_footnotelink', """If --device=paper, this option removes footnotes with links to computer programs."""),
    ('--latex_admon=', """Type of admonition in LaTeX: \n  colors1: (inspired by the NumPy User Guide) applies different colors for the different admons with an embedded icon; \n  colors2: like `colors1` but the text is wrapped around the icon; \n  mdfbox: rounded boxes with a optional title and no icon (default); \n  graybox2: box with square corners, gray background, and narrower than mdfbox, if code it reduces to something like mdfbox (mdframed based); the summary admon is in case of A4 format only half of the text width with text wrapped around (effective for proposals and articles); \n  grayicon: box with gray icons and a default light gray background; \n  yellowicon: box yellow icons and a default light yellow background; \n  paragraph:  plain paragraph with boldface heading. \n  Note: the colors in mdfbox and other boxes can customized."""),
    ('--latex_admon_color=', """The color to be used as background in admonitions. A single value applies to all admons: either rgb tuple (--latex_admon_color=0.1,0.1,0.4) or saturated color ('--latex_admon_color=yellow!5' - note the quotes needed for bash). \n  Multiple values can be assigned, one for each admon (all admons must be specified): '--latex_admon_color=warning:darkgreen!40!white;notice:darkgray!20!white;summary:tucorange!20!white;question:red!50!white;block:darkgreen!40!white' \
  If --latex_admon=mdfbox, the specification above with color1!X!color2 will automatically trigger 2*X as the background color of the frametitle.
  There are predefined multiple values, e.g., --latex_admon_color=colors1 gives red warnings, blue notice, orange questions, green summaries and yellow blocks, automatically adjusted with darker frametitles. \n  If --latex_admon=mdfbox, the background of the title and the color of the border of box can also be customized by direct editing. For example, a dark blue border and light blue title background is obtained by editing the .tex file as \n  doconce replace 'linecolor=black,' 'linecolor=darkblue,' mydoc.tex \n  doconce subst 'frametitlebackgroundcolor=.*?,' 'frametitlebackgroundcolor=blue!5,' mydoc.tex
  Actually, this particular (and common) edit is automatically done by the option --latex_admon_color=bluestyle
  --latex_admon_color=yellowstyle (the latter has color yellow!5 instead and yellow!20 for the border)"""),
    ('--latex_admon_title_no_period', """By default, a period is added to title admons that do not have a period, question mark, or similar. This option prevents adding a period such that the title acts like a heading."""),
    ('--latex_admon_envir_map=', """Mapping of code envirs to new envir names inside admons, e.g., to get a different code typesetting inside admons. This is useful if admons have a special color and the color background of code blocks does not fit will with the color background inside admons. Then it is natural to use a different verbatim code style inside admons. If specifying a number, say 2, as in --latex_admon_envir_map=2, an envir like pycod gets the number appended: pycod2. One can then in --latex_code_style= or in doconce ptex2tex or ptex2tex specify the typesetting of pycod2 environments. Otherwise the specification must be a mapping for each envir that should be changed inside the admons: --latex_admon_envir_map=pycod-pycod_yellow,fpro-fpro2 (from-to,from-to,... syntax)."""),
    ('--latex_subex_header_postfix=', """Default: ). Gives headers a), b), etc. Can be set to period, colon, etc."""),
    ('--xelatex', 'Use xelatex instead of latex/pdflatex.'),
    ('--latex_double_hyphen', """Replace single dash - by double dash -- in LaTeX output. Somewhat intelligent, but may give unwanted edits. Use with great care!"""),
    ('--latex_elsevier_journal=', """Sets the journal name for the --latex_style=elsevier style. Default: none (no journal name)."""),
    ('--ipynb_version=', 'ipynb version 3 (default) or 4.'),
    ('--ipynb_split_pyshell=', """Split interactive sessions into multiple cells after each output. Applies to pyshell and ipy code environments. on, True, yes: split (default). off, False, no: do not split. Note that pyshell-t and ipy-t environments just displays the session, while default pyshell and ipy removes all output (all output from print statements will come after the entire session)."""),
    ('--ipynb_disable_mpl_inline', """Disable automatic insertion of `%matplotlib inline` before the first import of matplotlib."""),
    ('--ipynb_cite=', """Typesetting of bibliography. plain: simple native typesetting (same as pandoc) (default); latex-plain: Similar to latex-style plain; latex: ipynb support for latex-style bibliographies (not mature)."""),
    ('--ipynb_admon=',  """Typesetting of admonitions (hint, remarks, box, notice, summary, warning, question, block - quotes are typeset as quotes). quote: as Markdown quote (default) with gray line on the left. paragraph: just the content with the title as paragraph heading. hrule: title with horizontal rule above and below, then text and horozontal rule."""),
    ('--ipynb_figure=', """How to typeset figures in ipynb: md (plain Markdown syntax); imgtag (<img src="..." width=...> tag, default); Image (python cell with Image object)."""),
    ('--ipynb_movie=', """How to typeset movies in ipynb: md (plain Markdown syntax, default); HTML: python cell with notebook `HTML` object containing the raw HTML code that is used in the DocOnce HTML format; ipynb: python cell with notebook `HTML` object with simple/standard ipynb HTML code for showing a YouTube or local video with a <video> tag."""),
    ('--verbose', 'Write out all OS commands run by doconce.'),
    ('--examples_as_exercises', """Treat examples of the form "==== Example: ..." as in exercise environments."""),
    ('--exercises_as_subsections', """Forces exercises to be typeset as subsections. Used to override various latex environments for exercises (esp. in Springer styles)."""),
    ('--solutions_at_end', 'Show solutions to exercises at the end of the document.'),
    ('--without_solutions', 'Leave out solution environments from exercises.'),
    ('--answers_at_end', 'Show answers to exercises at the end of the document.'),
    ('--without_answers', 'Leave out answer environments from exercises.'),
    ('--without_hints', 'Leave out hints from exercises.'),
    ('--exercise_solution=', 'Typesetting of solutions: paragraph, admon, or quote.'),
    ('--wordpress', 'Make HTML output for wordpress.com pages.'),
    ('--tables2csv', """Write each table to a CSV file table_X.csv, where X is the table number (autonumbered in according to appearance in the DocOnce source file)."""),
    ('--sections_up', """Upgrade all sections: sections to chapters, subsections to sections, etc."""),
    ('--sections_down', """Downgrade all sections: chapters to sections, sections to subsections, etc."""),
    ('--os_prompt=', """Terminal prompt in output from running OS commands (the @@@OSCMD instruction). None or empty: no prompt, just the command; nocmd: no command, just the output. Default is "Terminal>"."""),
    ('--code_skip_until=', '@@@CODE import: skip lines in files up to (and incuding) specified line.'),
    ('--code_prefix=', 'Prefix all @@@CODE imports with some path.'),
    ('--figure_prefix=', 'Prefix all figure filenames with, e.g., an URL.'),
    ('--movie_prefix=', 'Prefix all movie filenames with, e.g., an URL.'),
    ('--no_mp4_webm_ogg_alternatives', """Use just the specified (.mp4, .webm, .ogg) movie file; do not allow alternatives in HTML5 video tag. Used if the just the specified movie format should be played."""),
    ('--handout', 'Makes slides output suited for printing.'),
    ('--urlcheck', 'Check that all URLs referred to in the document are valid.'),
    ('--labelcheck=', 'Check that all ref{X} has a corresponding label{X}. Fake examples will fail this check and so will generalized references. Turn on when useful. Values: off (default), on.'),
    ('--short_title=', "Short version of the document's title."),
    ('--markdown', 'Allow Markdown (and some Extended Markdown) syntax as input.'),
    ('--md2do_output=', """Dump to file the DocOnce code arising from converting from Markdown. Default value is None (no dump). Any filename can be specified: --md2do_output=myfile.do.txt"""),
    ('--github_md', 'Turn on GitHub-flavored Markdown dialect of the pandoc translator'),
    ('--slate_md', 'Turn on Slate-extensions to Markdown in the pandoc translator. To be used together with --github_md.'),
    ('--strapdown', """Wrap Markdown output in HTML header/footer such that the output file (renamed as .html) can automatically be rendered as an HTML via strapdownjs.com technology. Combine with --github_md for richer output. Styles are set with --bootswatch_theme=cyborg (for instance)."""),
    ('--bootswatch_theme=', 'Bootswatch theme for use with --strapdown option.'),
    ('--strict_markdown_output', 'Ensure strict/basic Markdown as output.'),
    ('--multimarkdown_output', 'Allow MultiMarkdown as output.'),
    ('--quiz_question_prefix=', """Prefix/title before question in quizzes. Default: "Question:". Can also be set in square brackets for each individual question. ("Q: [] What is 1+1?" results in no prefix/title before the "What is 1+1?"."""),
    ('--quiz_choice_prefix=', """Prefix/title before choices in quizzes. \n  Default for HTML: "Choice", resulting in numbered choices "Choice 1:", "Choice 2:", etc. \n  A value with colon, period, or question mark (e.g., "Answer:") leaves out the numbering. \n  Default for latex/pdflatex: letter or letter+checkbox. \n  Other values: number, number+checkbox, number+circle, letter+circle, letter. \n  The checkbox or circle is always omitted if answers or solutions are included (i.e., if none of the --without_answers and --without_solutions is set). \n  The choice prefix can also be set in square brackets for each individual choice. ("Cr: [] Two" results in no prefix/title before the the answer "Two"."""),
    ('--quiz_horizontal_rule=', 'on (default): <hr> before and after quiz in HTML. off: no <hr>.'),
    ('--quiz_explanations=', """on/off. Some output formats do not support explanations with figures, math and/or code, this option turns all explanations off."""),
    ('--rst_uio', 'Univ. of Oslo version of rst files for their Vortex system.'),
    ('--rst_mathjax', 'Use raw HTML with MathJax for LaTeX mathematics in rst files.'),
    ('--sphinx_preserve_bib_keys', "Use the user's keys to in bibliography instead of numbers"),
    ('--sphinx_keep_splits', """Respect user's !split commands. Default: Override user's !split and insert new !split before all topmost sections. This is what makes sense in a Sphinx Table of Contents if one wants to split the document into multiple parts."""),
    ('--sphinx_figure_captions=', 'Font style in figure captions: emphasize (default) or normal. If you use boldface or emphasize in the caption, the font style will be normal for that caption.'),
    ('--oneline_paragraphs', 'Combine paragraphs to one line (does not work well).'),
    ('--html_responsive_figure_width', 'Use figure width as max-width, and set width to 100 percent so that figures can shrink to device width.')
    ]